/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "boitesdialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>

#include <math.h>

BoitesDialog::BoitesDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#ifndef Q_WS_MACX 
setModal(true);
#endif
connect(ui.quitButton, SIGNAL(clicked()), this, SLOT(accept()));
//setWindowFlags(Qt::Window|Qt::WindowMinMaxButtonsHint);
connect(ui.genererButton, SIGNAL(clicked()), this, SLOT(AfficherCode()));
connect(ui.listWidgetBoites, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditBoite, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterBoite()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerBoite()));
connect(ui.checkBoxAxes, SIGNAL(toggled(bool)),this, SLOT(ActiverAxes(bool)));
connect(ui.checkBoxGrille, SIGNAL(toggled(bool)),this, SLOT(ActiverGrille(bool)));


connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditMargeX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.nouveauButton, SIGNAL(clicked()), this, SLOT(NouvelleBoite()));
connect(ui.ouvrirButton, SIGNAL(clicked()), this, SLOT(ChargerBoite()));
connect(ui.sauverButton, SIGNAL(clicked()), this, SLOT(SauverBoite()));

connect(ui.aideButton, SIGNAL(clicked()), this, SLOT(Aide()));

connect(ui.apercuButton, SIGNAL(clicked()), this, SLOT(LatexTemp()));
connect(this, SIGNAL(latexfini()), this, SLOT(DvipsTemp()));
connect(this, SIGNAL(dvipsfini()), this, SLOT(BboxTemp()));
connect(this, SIGNAL(bboxfini()), this, SLOT(EpstopdfTemp()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));


connect(ui.epsButton, SIGNAL(clicked()), this, SLOT(ExporterEps()));
connect(ui.pdfButton, SIGNAL(clicked()), this, SLOT(ExporterPdf()));

ui.apercuButton->setShortcut(Qt::Key_F1);
ui.ouvrirButton->setShortcut(Qt::CTRL+Qt::Key_O);
ui.sauverButton->setShortcut(Qt::CTRL+Qt::Key_S);
ui.quitButton->setShortcut(Qt::CTRL+Qt::Key_Q);
ui.epsButton->setShortcut(Qt::CTRL+Qt::Key_E);
ui.pdfButton->setShortcut(Qt::CTRL+Qt::Key_P);
ui.apercuButton->setToolTip(QString::fromUtf8("F1"));
ui.ouvrirButton->setToolTip(QString::fromUtf8("CTRL+O"));
ui.sauverButton->setToolTip(QString::fromUtf8("CTRL+S"));
ui.quitButton->setToolTip(QString::fromUtf8("Quitter (CTRL+Q)"));
ui.epsButton->setToolTip(QString::fromUtf8("CTRL+E"));
ui.pdfButton->setToolTip(QString::fromUtf8("CTRL+P"));

ui.listWidgetBoites->setAlternatingRowColors(true);
ui.listWidgetBoites->setSelectionMode (QAbstractItemView::SingleSelection);

ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pstplus_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
texTempFile=prefixFile+".tex";
dviTempFile=prefixFile+".dvi";
psTempFile=prefixFile+".eps";
pdfTempFile=prefixFile+".pdf";
pngTempFile=prefixFile+".png";
txtTempFile=prefixFile+".txt";


highlighter = new Highlighter(ui.textEditCode->document());
highlighterSupp = new Highlighter(ui.textEditSupp->document());
QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined( Q_WS_MACX )
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_WS_WIN)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);


Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Diagrammes en boîtes : ")+nomFichier);
show();
}

BoitesDialog::~BoitesDialog(){
if (latexproc && latexproc->state()==QProcess::Running) 
	{
	latexproc->disconnect();
	latexproc->kill(); 
	delete latexproc ;
	}
if (dvipsproc && dvipsproc->state()==QProcess::Running) 
	{
	dvipsproc->disconnect();
	dvipsproc->kill(); 
	delete dvipsproc ;
	}
if (epstopdfproc && epstopdfproc->state()==QProcess::Running) 
	{
	epstopdfproc->disconnect();
	epstopdfproc->kill(); 
	delete epstopdfproc ;
	}
if (bboxproc && bboxproc->state()==QProcess::Running) 
	{
	bboxproc->disconnect();
	bboxproc->kill(); 
	delete bboxproc ;
	}
if (progressDialog) delete progressDialog;
}

void BoitesDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Diagrammes en boîtes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBoite();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void BoitesDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Diagrammes en boîtes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBoite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void BoitesDialog::Init()
{
ui.textEditCode->clear();
expressionList.clear();
couleurList.clear();
styleList.clear();
ui.listWidgetBoites->clear();
ui.checkBoxAxes->setChecked(true);
ui.checkBoxLabels->setChecked(true);
ui.checkBoxGrille->setChecked(false);
ui.lineEditUniteX->setText("1");
ui.lineEditXmin->setText("0");
ui.lineEditXmax->setText("1");
ui.lineEditMargeX->setText("0.1");
ui.lineEditGradX->setText("1");
ui.lineEditGrilleX->setText("1");
ui.lineEditGrilleX->setEnabled(false);
ui.textEditSupp->clear();
ActiverBoites(false);
}
//*********************************************
void BoitesDialog::ActiverBoites(bool etat)
{
ui.lineEditBoite->setEnabled(etat);
ui.comboBoxCouleur->setEnabled(etat);
ui.comboBoxStyle->setEnabled(etat);
//NettoyerCode();
}
void BoitesDialog::ActiverAxes(bool etat)
{
ui.lineEditGradX->setEnabled(etat);
ui.checkBoxLabels->setEnabled(etat);
NettoyerCode();
}
void BoitesDialog::ActiverGrille(bool etat)
{
ui.lineEditGrilleX->setEnabled(etat);
NettoyerCode();
}
//******************************************************************
QString BoitesDialog::GenererCode()
{
bool ok;

QString s_unitX=ui.lineEditUniteX->text();
double unitX=s_unitX.toDouble(&ok);

QString s_margeX=ui.lineEditMargeX->text();
double margeX=s_margeX.toDouble(&ok);

QString s_Xmin=ui.lineEditXmin->text();
double Xmin=s_Xmin.toDouble(&ok);

QString s_Xmax=ui.lineEditXmax->text();
double Xmax=s_Xmax.toDouble(&ok);

QString s_gradX=ui.lineEditGradX->text();
double gradX=s_gradX.toDouble(&ok);

QString s_grilleX=ui.lineEditGrilleX->text();
double grilleX=s_grilleX.toDouble(&ok);
if (grilleX<=0) grilleX=1;

QString s_unitY="1";
double unitY=1;
double grilleY=1;

QString s_Ymin="0";
double Ymin=0;

double Ymax=1.2*expressionList.count()+0.2;
QString s_Ymax=QString::number(Ymax);

double margeY=0.8;
QString s_margeY=QString::number(margeY);

double cadreXmin=Xmin-margeX;
double cadreXmax=Xmax+margeX;
double cadreYmin=Ymin-margeY;
double cadreYmax=Ymax+margeY;
double grilleXmin=Xmin/grilleX;
double grilleXmax=Xmax/grilleX;
double grilleYmin=Ymin;
double grilleYmax=Ymax;
double grilleUnitX=grilleX*unitX;
double grilleUnitY=Ymax-Ymin+1;

QString texcode="\\psset{xunit="+s_unitX+"cm , yunit="+s_unitY+"cm}\n";
texcode+="\\begin{pspicture*}("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
texcode+="\\def\\xmin{"+s_Xmin+"} "+"\\def\\xmax{"+s_Xmax+"} "+"\\def\\ymin{"+s_Ymin+"} "+"\\def\\ymax{"+s_Ymax+"}\n";
if (afficherCadre) texcode+="\\psframe[linewidth=0.3pt,linecolor=gray]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
else texcode+="\\newrgbcolor{couleurcadre}{0.99 0.99 0.99}\n\\psframe[linewidth=0.3pt,linecolor=couleurcadre]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
texcode+="\\def\\pshlabel#1{\\psframebox*[framesep=1pt]{\\small #1}}\n";
texcode+="\\def\\psvlabel#1{\\psframebox*[framesep=1pt]{\\small #1}}\n";

if (ui.checkBoxGrille->isChecked())
	{
texcode+="\\psset{xunit="+QString::number(grilleUnitX)+"cm,yunit="+QString::number(grilleUnitY)+"cm}\n";
texcode+="\\psgrid[griddots=25,gridlabels=0pt,gridwidth=.8pt, gridcolor=gray, subgridwidth=.8pt, subgridcolor=gray, subgriddiv=1](0,0)("+QString::number(grilleXmin)+","+QString::number(grilleYmin)+")("+QString::number(grilleXmax)+","+QString::number(grilleYmax)+")\n";
texcode+="\\psset{xunit="+s_unitX+"cm , yunit="+s_unitY+"cm}\n";
	}

int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
texcode+="\\newrgbcolor{couleur1}{"+QString::number(cR1 ,'g',5)+" "+QString::number(cG1 ,'g',5)+" "+QString::number(cB1 ,'g',5)+"}\n";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
texcode+="\\newrgbcolor{couleur2}{"+QString::number(cR2 ,'g',5)+" "+QString::number(cG2 ,'g',5)+" "+QString::number(cB2 ,'g',5)+"}\n";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
texcode+="\\newrgbcolor{couleur3}{"+QString::number(cR3 ,'g',5)+" "+QString::number(cG3 ,'g',5)+" "+QString::number(cB3 ,'g',5)+"}\n";

double base=0;
QStringList val;
QString couleur,style;
for (int i = 0; i < expressionList.count(); i++)
	{
	base+=1.2;
	val=expressionList.at(i).split(";");
	if (val.count()==5)
		{
		if (couleurList.at(i)=="noir") couleur="black";
		else if (couleurList.at(i)=="gris") couleur="gray";
		else if (couleurList.at(i)=="gris foncé") couleur="darkgray";
		else if (couleurList.at(i)=="gris clair") couleur="lightgray";
		else if (couleurList.at(i)=="bleu") couleur="blue";
		else if (couleurList.at(i)=="rouge") couleur="red";
		else if (couleurList.at(i)=="vert") couleur="green";
		else if (couleurList.at(i)=="jaune") couleur="yellow";
		else if (couleurList.at(i)=="cyan") couleur="cyan";
		else if (couleurList.at(i)=="magenta") couleur="magenta";
		else if (couleurList.at(i)=="couleur1") couleur="couleur1";
		else if (couleurList.at(i)=="couleur2") couleur="couleur2";
		else if (couleurList.at(i)=="couleur3") couleur="couleur3";
		else couleur="black";
		if (styleList.at(i)=="ligne") style="solid";
		else if (styleList.at(i)=="trait") style="dashed";
		else if (styleList.at(i)=="point") style="dotted";
		else style="solid";
		texcode+="\\psframe[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]("+val.at(1)+","+QString::number(base-0.4)+")("+val.at(3)+","+QString::number(base+0.4)+")\n";
		texcode+="\\psline[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]{-}("+val.at(2)+","+QString::number(base-0.4)+")("+val.at(2)+","+QString::number(base+0.4)+")\n";
		texcode+="\\psline[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]{-}("+val.at(0)+","+QString::number(base)+")("+val.at(1)+","+QString::number(base)+")\n";
		texcode+="\\psline[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]{-}("+val.at(3)+","+QString::number(base)+")("+val.at(4)+","+QString::number(base)+")\n";
		texcode+="\\psline[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]{-}("+val.at(0)+","+QString::number(base-0.15)+")("+val.at(0)+","+QString::number(base+0.15)+")\n";
		texcode+="\\psline[linewidth=1.2pt,linecolor="+couleur+",linestyle="+style+"]{-}("+val.at(4)+","+QString::number(base-0.15)+")("+val.at(4)+","+QString::number(base+0.15)+")\n";
		}
	else
		{
		texcode+="\\uput[dl](\\xmin,"+QString::number(base)+"){Erreur}\n";
		}
	}
texcode+=ui.textEditSupp->toPlainText()+"\n";
if (ui.checkBoxAxes->isChecked())
	{
	if (ui.checkBoxLabels->isChecked()) texcode+="\\psaxes[labels=all,labelsep=1pt,Dx="+s_gradX+",Ox=\\xmin,Oy=\\ymin]{-}(\\xmin,0)(\\xmin,0)(\\xmax,0.1)\n";
	else texcode+="\\psaxes[labels=none,labelsep=1pt,Dx="+s_gradX+",Ox=\\xmin,Oy=\\ymin]{-}(\\xmin,\\ymin)(\\xmin,\\ymin)(\\xmax,\\ymin)\n";
	}
texcode+="\\end{pspicture*}\n";
return texcode;
}

void BoitesDialog::AfficherCode()
{
QString texcode="%\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void BoitesDialog::ajouterBoite()
{
ActiverBoites(true);
expressionList.append("Nouveau");
couleurList.append("noir");
styleList.append("ligne");
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetBoites);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetBoites->setCurrentItem(nouveauItem);
ui.listWidgetBoites->setItemSelected(ui.listWidgetBoites->currentItem(), true);
ui.lineEditBoite->setFocus();
NettoyerCode();
}

void BoitesDialog::supprimerBoite()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetBoites->currentItem();
expressionList.removeAt(courant);
couleurList.removeAt(courant);
styleList.removeAt(courant);
ui.listWidgetBoites->setCurrentItem(ui.listWidgetBoites->item(courant));
ui.listWidgetBoites->setItemSelected(ui.listWidgetBoites->item(courant),true);
actualiserItemInfo();
NettoyerCode();
}

void BoitesDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetBoites->item(courant)->setText(ui.lineEditBoite->text());
expressionList[courant]=ui.lineEditBoite->text();
NettoyerCode();
}

void BoitesDialog::actualiserCouleur()
{
int courant=numeroItem();
if (courant<0) return;
couleurList[courant]=ui.comboBoxCouleur->currentText();
NettoyerCode();
}

void BoitesDialog::actualiserStyle()
{
int courant=numeroItem();
if (courant<0) return;
styleList[courant]=ui.comboBoxStyle->currentText();
NettoyerCode();
}

void BoitesDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditBoite->setText("");
	ui.comboBoxCouleur->setCurrentIndex(0);
	ui.comboBoxStyle->setCurrentIndex(0);
	ActiverBoites(false);
	}
else
	{
	ui.lineEditBoite->disconnect(SIGNAL(textChanged(const QString &)));
	ui.comboBoxCouleur->disconnect(SIGNAL(currentIndexChanged(const QString &)));
	ui.comboBoxStyle->disconnect(SIGNAL(currentIndexChanged(const QString &)));

	ui.lineEditBoite->setText(expressionList[courant]);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(couleurList[courant],Qt::MatchExactly));
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(styleList[courant],Qt::MatchExactly));

	connect(ui.lineEditBoite, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
	connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
	ActiverBoites(true);
	}
}

int BoitesDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetBoites->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetBoites->setCurrentItem(items[0]);
	return ui.listWidgetBoites->row(items[0]);
	}
return -1;
}
//******************************************************
void BoitesDialog::NouvelleBoite()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Diagrammes en boîtes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBoite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Diagrammes en boîtes : ")+nomFichier);
}

void BoitesDialog::SauverBoite()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Boites");
doc.appendChild(root);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("MargeX");
contenu=doc.createTextNode(ui.lineEditMargeX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleX");
contenu=doc.createTextNode(ui.lineEditGrilleX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Axes");
if (ui.checkBoxAxes->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Grille");
if (ui.checkBoxGrille->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Valeurs");
contenu=doc.createTextNode(expressionList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleurs");
contenu=doc.createTextNode(couleurList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Styles");
contenu=doc.createTextNode(styleList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Diagramme (*.pst)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Diagramme (*.pst)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Diagrammes en boîtes : ")+nomFichier);
}

void BoitesDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Boites")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à des courbes"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("MargeX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditMargeX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Axes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAxes->setChecked(true);
	else ui.checkBoxAxes->setChecked(false);
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Grille");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxGrille->setChecked(true);
	else ui.checkBoxGrille->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Valeurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	expressionList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Couleurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	couleurList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Styles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	styleList=node.toElement().text().split("#");
	}
fichier.close();
ui.listWidgetBoites->clear();
for (int i = 0; i < expressionList.count(); i++)
	{
	if (!expressionList.at(i).isEmpty() && !couleurList.at(i).isEmpty() && !styleList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetBoites);
		nouveauItem->setText(expressionList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		expressionList.removeAt(i);
		couleurList.removeAt(i);
		styleList.removeAt(i);
		}
	}
if (ui.listWidgetBoites->count()>0)
	{
	ui.listWidgetBoites->setCurrentItem(ui.listWidgetBoites->item(0));
	ui.listWidgetBoites->setItemSelected(ui.listWidgetBoites->currentItem(), true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Diagrammes en boîtes : ")+nomFichier);
}

void BoitesDialog::ChargerBoite()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Diagrammes en boîtes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en charger un autre?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverBoite();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Diagramme (*.pst)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void BoitesDialog::LatexTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+texTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << "\\documentclass[10pt,a4paper]{article}\n";
out << "\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
out << "\\usepackage{amsmath}\n\\usepackage{amsfonts}\n\\usepackage{amssymb}\n";
out << "\\pagestyle{empty}\n";
out << "\\begin{document}\n";
out << "\\TeXtoEPS\n";
out << GenererCode();
out << "\\endTeXtoEPS\n";
out << "\\end{document}\n";
fichier.close();
erreurProcessus=false;
latexproc = new QProcess( this );
latexproc->setWorkingDirectory(tempDir);
connect(latexproc, SIGNAL(finished(int)),this, SLOT(finLatexProcess(int)));
latexproc->start(latexCommande+" -interaction=nonstopmode "+texTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(QString::fromUtf8("Compilation en cours..."),0,0,4,this);
progressDialog->show();
progressDialog->raise();
progressDialog->setValue(0);
}

void BoitesDialog::finLatexProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	LogDialog *logDlg = new LogDialog(this);
	logDlg->exec();
	//QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation LaTeX"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(1);
	emit latexfini();
	}
}

void BoitesDialog::DvipsTemp()
{
dvipsproc = new QProcess( this );
dvipsproc->setWorkingDirectory(tempDir);
connect(dvipsproc, SIGNAL(finished(int)),this, SLOT(finDvipsProcess(int)));
dvipsproc->start(dvipsCommande+" -Ppk -V -E -o "+psTempFile+" "+dviTempFile);
}

void BoitesDialog::finDvipsProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande dvips"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(2);
	emit dvipsfini();
	}
}

void BoitesDialog::BboxTemp()
{
bboxproc = new QProcess( this );
bboxproc->setWorkingDirectory(tempDir);
connect(bboxproc, SIGNAL(finished(int)),this, SLOT(finBboxProcess(int)));
connect(bboxproc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
bboxproc->start(gsCommande+" -q -dBATCH -dNOPAUSE -sDEVICE=bbox "+psTempFile);
}

void BoitesDialog::readInfoOutput()
{
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QString line;
QByteArray result=bboxproc->readAllStandardError();
QTextStream bbox(&result);
while ( !bbox.atEnd() ) 
	{
	line=bbox.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		int nx0=x0.toInt()-5;
		int nx1=x1.toInt()+5;
		int ny0=y0.toInt()-5;
		int ny1=y1.toInt()+5;
		QFile psf(tempDir+"/"+psTempFile);
		if (!psf.open( QIODevice::ReadOnly )) 
			{
			erreurProcessus=true;
			return;
			}
		QTextStream tpsf( &psf );
		line=tpsf.readAll();
		psf.close();
		line.replace(rxbbox,"%%BoundingBox: "+QString::number(nx0)+" "+QString::number(ny0)+" "+QString::number(nx1)+" "+QString::number(ny1));
		if (!psf.open(QIODevice::WriteOnly))
			{
			erreurProcessus=true;
			return;
			}
		QTextStream out( &psf );
		out << line;
		psf.close();
		emit bboxfini();
		}
	}
}


void BoitesDialog::finBboxProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande bbox"));
	erreurProcessus=true;
	}
else if (progressDialog) progressDialog->setValue(3);
}

void BoitesDialog::EpstopdfTemp()
{
bool ok;
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QFile psf(tempDir+"/"+psTempFile);
if (!psf.open( QIODevice::ReadOnly )) return;
QTextStream tpsf( &psf );
QString line;
while ( !tpsf.atEnd() ) 
	{
	line=tpsf.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		}
	}
psf.close();
float nx0=x0.toFloat(&ok);
float nx1=x1.toFloat(&ok);
if (nx1<=nx0) return;
float ny0=y0.toFloat(&ok);
float ny1=y1.toFloat(&ok);

epstopdfproc = new QProcess( this );
epstopdfproc->setWorkingDirectory(tempDir);
connect(epstopdfproc, SIGNAL(finished(int)),this, SLOT(finEpstopdfProcess(int)));
//epstopdfproc->start(epstopdfCommande+" "+psTempFile);
QString command=gsCommande+" -q -dBATCH -dNOPAUSE -dUseFlateCompression=false -sDEVICE=pdfwrite -sOutputFile="+pdfTempFile+" -c \"<< /PageSize ["+QString::number(nx1-nx0)+" "+QString::number(ny1-ny0)+"] >> setpagedevice\" "+QString::number(0-nx0)+" "+QString::number(0-ny0)+" translate -f "+psTempFile;
epstopdfproc->start(command);
}

void BoitesDialog::finEpstopdfProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande epstopdf"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(4);
	emit epstopdffini();
	}
}


void BoitesDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void BoitesDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Diagrammes en boîtes : ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void BoitesDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"boites");
aideDialog->exec();
}
//*****************************
void BoitesDialog::ExporterEps()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterEpsSuite()));
LatexTemp();
}

void BoitesDialog::ExporterEpsSuite()
{
if (!erreurProcessus)
	{
	QString epsFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Eps (*.eps)"));
	if (!epsFichier.isEmpty())
		{
		QFileInfo fi(epsFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+psTempFile,epsFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void BoitesDialog::ExporterPdf()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterPdfSuite()));
LatexTemp();
}

void BoitesDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void BoitesDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

