/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "courbesdialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>

#include <math.h>

CourbesDialog::CourbesDialog(QWidget *parent,bool use_small)
    :QDialog( parent)
{
if (use_small) smallcode="\\small";
else smallcode="";
ui.setupUi(this);

#ifndef Q_WS_MACX 
setModal(true);
#endif
connect(ui.quitButton, SIGNAL(clicked()), this, SLOT(accept()));
//setWindowFlags(Qt::Window|Qt::WindowMinMaxButtonsHint);
connect(ui.genererButton, SIGNAL(clicked()), this, SLOT(AfficherCode()));
connect(ui.listWidgetFonctions, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditFonction, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.lineEditVarmin, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserVarmin()));
connect(ui.lineEditVarmax, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserVarmax()));
connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterFonction()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerFonction()));
connect(ui.checkBoxAxes, SIGNAL(toggled(bool)),this, SLOT(ActiverAxes(bool)));
connect(ui.checkBoxGrille, SIGNAL(toggled(bool)),this, SLOT(ActiverGrille(bool)));

connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditMargeX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditUniteY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditMargeY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.checkBoxRepere, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));
connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));

connect(ui.nouveauButton, SIGNAL(clicked()), this, SLOT(NouvelleCourbe()));
connect(ui.ouvrirButton, SIGNAL(clicked()), this, SLOT(ChargerCourbe()));
connect(ui.sauverButton, SIGNAL(clicked()), this, SLOT(SauverCourbe()));


connect(ui.effacerButton, SIGNAL(clicked()), this, SLOT(NettoyerSupp()));

connect(ui.aideButton, SIGNAL(clicked()), this, SLOT(Aide()));
connect(ui.calculerButton, SIGNAL(clicked()), this, SLOT(AfficherCalcul()));

connect(ui.apercuButton, SIGNAL(clicked()), this, SLOT(LatexTemp()));
connect(this, SIGNAL(latexfini()), this, SLOT(DvipsTemp()));
connect(this, SIGNAL(dvipsfini()), this, SLOT(BboxTemp()));
connect(this, SIGNAL(bboxfini()), this, SLOT(EpstopdfTemp()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));


connect(ui.epsButton, SIGNAL(clicked()), this, SLOT(ExporterEps()));
connect(ui.pdfButton, SIGNAL(clicked()), this, SLOT(ExporterPdf()));

ui.apercuButton->setShortcut(Qt::Key_F1);
ui.ouvrirButton->setShortcut(Qt::CTRL+Qt::Key_O);
ui.sauverButton->setShortcut(Qt::CTRL+Qt::Key_S);
ui.quitButton->setShortcut(Qt::CTRL+Qt::Key_Q);
ui.epsButton->setShortcut(Qt::CTRL+Qt::Key_E);
ui.pdfButton->setShortcut(Qt::CTRL+Qt::Key_P);
ui.apercuButton->setToolTip(QString::fromUtf8("F1"));
ui.ouvrirButton->setToolTip(QString::fromUtf8("CTRL+O"));
ui.sauverButton->setToolTip(QString::fromUtf8("CTRL+S"));
ui.quitButton->setToolTip(QString::fromUtf8("Quitter (CTRL+Q)"));
ui.epsButton->setToolTip(QString::fromUtf8("CTRL+E"));
ui.pdfButton->setToolTip(QString::fromUtf8("CTRL+P"));

ui.listWidgetFonctions->setAlternatingRowColors(true);
ui.listWidgetFonctions->setSelectionMode (QAbstractItemView::SingleSelection);

ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pstplus_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
texTempFile=prefixFile+".tex";
dviTempFile=prefixFile+".dvi";
psTempFile=prefixFile+".eps";
pdfTempFile=prefixFile+".pdf";
pngTempFile=prefixFile+".png";
txtTempFile=prefixFile+".txt";

itemMacros << QString::fromUtf8("Point A(x,y)");
codeMacros << "\\psdot(x,y) \\uput[ur](x,y){A}\n";
itemMacros << QString::fromUtf8("Point A(x,y) + axes");
codeMacros << "\\psdot(x,y) \\uput[ur](x,y){A}\n\\psline[linestyle=dashed](x,0)(x,y)(0,y)\n";
itemMacros << QString::fromUtf8("Asymptote verticale x=a");
codeMacros << "\\psline[linestyle=solid,linecolor=black](a,\\ymin)(a,\\ymax)\n";
itemMacros << QString::fromUtf8("Aire sous courbe [a,b]");
codeMacros << "\\pscustom[fillstyle=hlines,hatchwidth=0.3pt,hatchsep=12pt,hatchcolor=gray,linestyle=solid]\n{\n\\psplot{a}{b}{\\F}\n\\psline[linestyle=solid,linecolor=gray](b,0)(a,0)(a,f(a))\n}\n";

ui.comboBoxMacros->addItems(itemMacros);
connect(ui.comboBoxMacros, SIGNAL(activated(int)), this, SLOT(AjouterMacro(int)));

highlighter = new Highlighter(ui.textEditCode->document());
highlighterSupp = new Highlighter(ui.textEditSupp->document());
QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined( Q_WS_MACX )
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_WS_WIN)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Courbes : ")+nomFichier);
show();
}

CourbesDialog::~CourbesDialog(){
if (latexproc && latexproc->state()==QProcess::Running) 
	{
	latexproc->disconnect();
	latexproc->kill(); 
	delete latexproc ;
	}
if (dvipsproc && dvipsproc->state()==QProcess::Running) 
	{
	dvipsproc->disconnect();
	dvipsproc->kill(); 
	delete dvipsproc ;
	}
if (epstopdfproc && epstopdfproc->state()==QProcess::Running) 
	{
	epstopdfproc->disconnect();
	epstopdfproc->kill(); 
	delete epstopdfproc ;
	}
if (bboxproc && bboxproc->state()==QProcess::Running) 
	{
	bboxproc->disconnect();
	bboxproc->kill(); 
	delete bboxproc ;
	}
if (progressDialog) delete progressDialog;
}

void CourbesDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Courbes : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCourbe();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void CourbesDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Courbes : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCourbe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void CourbesDialog::Init()
{
ui.textEditCode->clear();
expressionList.clear();
varminList.clear();
varmaxList.clear();
couleurList.clear();
styleList.clear();
ui.listWidgetFonctions->clear();
ui.checkBoxAxes->setChecked(true);
ui.checkBoxLabels->setChecked(true);
ui.checkBoxRepere->setChecked(true);
ui.checkBoxGrille->setChecked(true);
ui.lineEditUniteX->setText("1");
ui.lineEditXmin->setText("0");
ui.lineEditXmax->setText("1");
ui.lineEditMargeX->setText("0.1");
ui.lineEditGradX->setText("1");
ui.lineEditGrilleX->setText("1");
ui.lineEditUniteY->setText("1");
ui.lineEditYmin->setText("0");
ui.lineEditYmax->setText("1");
ui.lineEditMargeY->setText("0.1");
ui.lineEditGradY->setText("1");
ui.lineEditGrilleY->setText("1");
ui.textEditSupp->clear();
ActiverFonctions(false);
}
//*********************************************
void CourbesDialog::ActiverFonctions(bool etat)
{
ui.lineEditFonction->setEnabled(etat);
ui.lineEditVarmin->setEnabled(etat);
ui.lineEditVarmax->setEnabled(etat);
ui.comboBoxCouleur->setEnabled(etat);
ui.comboBoxStyle->setEnabled(etat);
//NettoyerCode();
}
void CourbesDialog::ActiverAxes(bool etat)
{
ui.lineEditGradX->setEnabled(etat);
ui.lineEditGradY->setEnabled(etat);
ui.checkBoxLabels->setEnabled(etat);
NettoyerCode();
}
void CourbesDialog::ActiverGrille(bool etat)
{
ui.lineEditGrilleX->setEnabled(etat);
ui.lineEditGrilleY->setEnabled(etat);
NettoyerCode();
}
//******************************************************************
bool CourbesDialog::Operateur(QString mot)
{
bool result=false;
if ( (mot=="(") || (mot==")") || (mot=="-") || (mot=="+") || (mot=="*") || (mot=="/") || (mot=="^") ) result=true;
return result;
}

bool CourbesDialog::Fonction(QString mot)
{
bool result=false;
if ( (mot=="cos") || (mot=="sin") || (mot=="rac") || (mot=="ln") || (mot=="log") || (mot=="exp") || (mot=="tan") || (mot=="abs") || (mot=="acos") || (mot=="asin") || (mot=="atan") ) result=true;
return result;
}

int CourbesDialog::Priorite(QString mot)
{
int result=-1;
if (mot=="(") result=0;
else if (mot==")") result=1;
else if (mot=="-") result=2;
else if (mot=="+") result=2;
else if (mot=="*") result=3;
else if (mot=="/") result=3;
else if (mot=="^") result=5;
else if (Fonction(mot)) result=4;
return result;
}

QString CourbesDialog::CalculOp(QString n,QString m,QString x,QString variable)
{
double calcul=0;
double p,q;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (m=="x") q=variable.toDouble(&ok);
else q=m.toDouble(&ok);
if (x=="+") calcul=q+p;
if (x=="-") calcul=q-p;
if (x=="*") calcul=q*p;
if (x=="/") 
	{
	if (p!=0) calcul=q/p;
	else existvaleur=false;
	}
if (x=="^") 
{
	if ((floor(abs(p))==(abs(p))) && (p>=0)) 
		{
		if (p==0) calcul=1;
		if (p==1) calcul=q;
		if (p>1) 
			{
			calcul=1;
			for (int i = 1; i <=p; i++) calcul=calcul*q;
			}
		}
	else 
		{
		if (q>1e-4) calcul=exp(p*log(abs(q)));
		else existvaleur=false;
		}
}
return QString::number(calcul,'g',12);
}

QString CourbesDialog::CalculAp(QString n,QString x,QString variable)
{
double calcul=0;
double p;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (x=="cos")  calcul=cos(p);
if (x=="sin")  calcul=sin(p);
if (x=="abs")  calcul=abs(p);
if (x=="rac") 
   {
   if (p>=0)  calcul=sqrt(p);
   else existvaleur=false;
   }
if (x=="ln") 
   {
   if (p>1e-50)  calcul=log(p);
   else existvaleur=false;
   }
if (x=="exp") 
   {
   if ((p<80) && (p>-80))  calcul=exp(p);
   else existvaleur=false;
   }
if (x=="tan") 
   {
   if (cos(p)!=0) calcul=tan(p);
   else existvaleur=false;
   }
if (x=="acos") 
   {
   if ((p<=1) && (p>=-1)) calcul=acos(p);
   else existvaleur=false;
   }
if (x=="asin") 
   {
   if ((p<=1) && (p>=-1)) calcul=asin(p);
   else existvaleur=false;
   }
if (x=="atan") calcul=atan(p);
if (x=="log") 
   {
   if (p>1e-50)  calcul=log10(p);
   else existvaleur=false;
   }
return QString::number(calcul,'g',12);
}


QString CourbesDialog::CalculerExpression(QString expression, QString variable)
{
QStringList Pp,Rp,Ep,Cp;
Pp.clear();
Rp.clear();
Ep.clear();
Cp.clear();
QString lettre,mot,n,m,z,result;
//**** PARSEUR et CREATION PILES
int i;
bool fini;
if (expression.isEmpty()) return result;
expression+="f";
i=0;
fini=false;
do
	{
	lettre=expression.mid(i,1);
	i+=1;
	if (lettre=="-")
		{
		if (Ep.isEmpty()) Ep.append("0");
		else if (Ep.last()=="(") Ep.append("0");
		}
	if (Operateur(lettre) || lettre=="x") Ep.append(lettre);
	else
		{
		mot="";
		while ((!Operateur(lettre)) && (lettre!="f")) 
			{
			mot+=lettre;
			lettre=expression.mid(i,1);
			i+=1;
			}
		if (mot=="pi") mot="3.141592653589793238";
		if (!mot.isEmpty()) Ep.append(mot);
		if (lettre=="f") 
			{
			fini=true;
			Ep.append("f");
			}
		else Ep.append(lettre);
		}
	}
while (!fini);
//******* POSTFIXE
i=0;
mot="";
Pp.append("(");
mot=Ep.at(i);
while (mot!="f") 
	{
	if ((!Operateur(mot)) && (!Fonction(mot))) Rp.append(mot);
	else
		{
		if (mot=="(") Pp.append(mot);
		else
			{
			while (Priorite(Pp.last())>=Priorite(mot)) 
				{
				Rp.append(Pp.last());
				Pp.removeLast();
				}
			if (mot==")")  Pp.removeLast();
			if (mot!=")")  Pp.append(mot);
			}
		}
	i++;
	mot=Ep.at(i);
	}
while (Pp.last()!="(" && !Pp.isEmpty())
      {
      Rp.append(Pp.last());
      Pp.removeLast();
      }
if (Rp.isEmpty()) Rp.append("0");
//******* CALCULER
existvaleur=true;
for (int ind=0;ind<Rp.count();ind++) 
	{
	mot=Rp.at(ind);
	if (Operateur(mot)) 
		{
		n=Cp.last();
		Cp.removeLast();
		m=Cp.last();
		Cp.removeLast();
		z=CalculOp(n,m,mot,variable);
		Cp.append(z);
		}
	else
	{
	if (Fonction(mot))
		{
		n=Cp.last();
		Cp.removeLast();
		z=CalculAp(n,mot,variable);
		Cp.append(z);
		}
	else 
		{
		Cp.append(mot);
		}
	}
	}
if (Cp.first()=="x") result=variable;
else result=Cp.first();
if (!existvaleur) result="Erreur";
return result;
}

QString CourbesDialog::CalculerExpressionArrondi(QString expression, QString variable)
{
QString result;
double num_result, num_resultbis;
int approx_result;
bool ok;
result=CalculerExpression(expression,variable);
if (result!="Erreur") 
	{
	num_result=result.toDouble(&ok);
	if (ok) 
		{
		num_resultbis=floor (num_result*1e5);
		approx_result=(int) num_resultbis;
		num_result=(double) (approx_result/1e5);
		result=QString::number(num_result,'g',12);
		}
	}
return result;
}

void CourbesDialog::AfficherCalcul()
{
if (!ui.lineEditFonction->isEnabled()) return;
bool ok;
QString f,text_a,text_ah,text_fa,text_fah,text_der,text_cte;
double num_a,num_ah,num_fa,num_fah,num_der,num_cte;

f=ui.lineEditFonction->text();

text_a=CalculerExpression(ui.lineEditVariable->text(),"1");
if (text_a=="Erreur")
	{
	ui.lineEditVariable->setText("Erreur");
	return;
	}
num_a=text_a.toDouble(&ok);

if (f.isEmpty() || text_a.isEmpty()) return;

text_fa=CalculerExpression(f,text_a);
num_fa=text_fa.toDouble(&ok);

if (text_fa!="Erreur") ui.lineEditValeur->setText(QString::number(num_fa,'g',6));
else ui.lineEditValeur->setText("Erreur");

text_ah=QString::number(num_a+0.000001,'g',12);
num_ah=text_ah.toDouble(&ok);
text_fah=CalculerExpression(f,text_ah);
num_fah=text_fah.toDouble(&ok);
if (text_fa!="Erreur" && text_fah!="Erreur")
	{
	num_der=(num_fah-num_fa)/0.000001;
	text_der=QString::number(num_der,'g',6);
	ui.lineEditDerive->setText(text_der);
	num_cte=num_fa-num_der*num_a;
	text_cte=QString::number(num_cte,'g',6);
	ui.lineEditTangente->setText("y="+text_der+"*x+("+text_cte+")");
	}
else
	{
	ui.lineEditDerive->setText("Erreur");
	ui.lineEditTangente->setText("Erreur");
	}
}

QString CourbesDialog::Postfixe(QString expression)
{
QStringList Pp,Rp,Ep;
Pp.clear();
Rp.clear();
Ep.clear();
QString lettre,mot;
QString result="erreur";
int i;
bool fini;
if (expression.isEmpty()) return result;
expression+="f";
i=0;
fini=false;
do
	{
	lettre=expression.mid(i,1);
	i+=1;
	if (lettre=="-")
		{
		if (Ep.isEmpty()) Ep.append("0");
		else if (Ep.last()=="(") Ep.append("0");
		}
	if (Operateur(lettre) || lettre=="x") Ep.append(lettre);
	else
		{
		mot="";
		while ((!Operateur(lettre)) && (lettre!="f")) 
			{
			mot+=lettre;
			lettre=expression.mid(i,1);
			i+=1;
			}
		if (mot=="pi") mot="3.141592653589793238";
		if (!mot.isEmpty()) Ep.append(mot);
		if (lettre=="f") 
			{
			fini=true;
			Ep.append("f");
			}
		else Ep.append(lettre);
		}
	}
while (!fini);
i=0;
mot="";
Pp.append("(");
mot=Ep.at(i);
while (mot!="f") 
	{
	if ((!Operateur(mot)) && (!Fonction(mot))) Rp.append(mot);
	else
		{
		if (mot=="(") Pp.append(mot);
		else
			{
			while (Priorite(Pp.last())>=Priorite(mot)) 
				{
				Rp.append(Pp.last());
				Pp.removeLast();
				}
			if (mot==")")  Pp.removeLast();
			if (mot!=")")  Pp.append(mot);
			}
		}
	i++;
	mot=Ep.at(i);
	}
while (Pp.last()!="(" && !Pp.isEmpty())
      {
      Rp.append(Pp.last());
      Pp.removeLast();
      }
if (Rp.isEmpty()) Rp.append("0");
result =Rp.join(" ");
result.replace(QString(" exp"),QString(" EXP"));
result.replace(QString(" rac"),QString(" sqrt"));
result.replace(QString(" sin"),QString(" SIN"));
result.replace(QString(" cos"),QString(" COS"));
result.replace(QString(" tan"),QString(" TAN"));
result.replace(QString(" acos"),QString(" ACOS"));
result.replace(QString(" asin"),QString(" ASIN"));
result.replace(QString(" atan"),QString(" ATAN"));
result.replace(QString(" +"),QString(" add"));
result.replace(QString(" -"),QString(" sub"));
result.replace(QString(" *"),QString(" mul"));
result.replace(QString(" /"),QString(" div"));
result.replace(QString(" ^"),QString(" exp"));
return result;
}

QString CourbesDialog::GenererCode()
{
QString couleur,style, s_varmin, s_varmax;

bool ok;

QString s_unitX=CalculerExpressionArrondi(ui.lineEditUniteX->text(),"1");
double unitX;
if (s_unitX!="Erreur") unitX=s_unitX.toDouble(&ok);
else unitX=1;

QString s_unitY=CalculerExpressionArrondi(ui.lineEditUniteY->text(),"1");
double unitY;
if (s_unitY!="Erreur") unitY=s_unitY.toDouble(&ok);
else unitY=1;

QString s_margeX=CalculerExpressionArrondi(ui.lineEditMargeX->text(),"1");
double margeX;
if (s_margeX!="Erreur") margeX=s_margeX.toDouble(&ok);
else margeX=0.1;

QString s_margeY=CalculerExpressionArrondi(ui.lineEditMargeY->text(),"1");
double margeY;
if (s_margeY!="Erreur") margeY=s_margeY.toDouble(&ok);
else margeY=0.1;

QString s_Xmin=CalculerExpressionArrondi(ui.lineEditXmin->text(),"1");
double Xmin;
if (s_Xmin!="Erreur") Xmin=s_Xmin.toDouble(&ok);
else Xmin=0;

QString s_Ymin=CalculerExpressionArrondi(ui.lineEditYmin->text(),"1");
double Ymin;
if (s_Ymin!="Erreur") Ymin=s_Ymin.toDouble(&ok);
else Ymin=0;

QString s_Xmax=CalculerExpressionArrondi(ui.lineEditXmax->text(),"1");
double Xmax;
if (s_Xmax!="Erreur") Xmax=s_Xmax.toDouble(&ok);
else Xmax=1;

QString s_Ymax=CalculerExpressionArrondi(ui.lineEditYmax->text(),"1");
double Ymax;
if (s_Ymax!="Erreur") Ymax=s_Ymax.toDouble(&ok);
else Ymax=1;

QString s_gradX=CalculerExpressionArrondi(ui.lineEditGradX->text(),"1");
double gradX;
if (s_gradX!="Erreur") gradX=s_gradX.toDouble(&ok);
else gradX=1;

QString s_gradY=CalculerExpressionArrondi(ui.lineEditGradY->text(),"1");
double gradY;
if (s_gradY!="Erreur") gradY=s_gradY.toDouble(&ok);
else gradY=1;

QString s_grilleX=CalculerExpressionArrondi(ui.lineEditGrilleX->text(),"1");
double grilleX;
if (s_grilleX!="Erreur") grilleX=s_grilleX.toDouble(&ok);
else grilleX=1;
if (grilleX<=0) grilleX=1;

QString s_grilleY=CalculerExpressionArrondi(ui.lineEditGrilleY->text(),"1");
double grilleY;
if (s_grilleY!="Erreur") grilleY=s_grilleY.toDouble(&ok);
else grilleY=1;
if (grilleY<=0) grilleY=1;

double cadreXmin=Xmin-margeX;
double cadreXmax=Xmax+margeX;
double cadreYmin=Ymin-margeY;
double cadreYmax=Ymax+margeY;
double grilleXmin=Xmin/grilleX;
double grilleXmax=Xmax/grilleX;
double grilleYmin=Ymin/grilleY;
double grilleYmax=Ymax/grilleY;
double grilleUnitX=grilleX*unitX;
double grilleUnitY=grilleY*unitY;
// int pe=(int) abs(Xmin)/grilleX;
// double debutgrilleX=(double) ((abs(Xmin)/grilleX)-pe)*grilleX+Xmin;
// pe=(int) abs(Ymin)/grilleY;
// double debutgrilleY=(double) ((abs(Ymin)/grilleY)-pe)*grilleY+Ymin;
// int nblignesX=(int) (Xmax-Xmin)/grilleX;
// int nblignesY=(int) (Ymax-Ymin)/grilleY;
QString texcode="\\psset{xunit="+s_unitX+"cm , yunit="+s_unitY+"cm}\n";
texcode+="\\begin{pspicture*}("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
texcode+="\\def\\xmin{"+s_Xmin+"} "+"\\def\\xmax{"+s_Xmax+"} "+"\\def\\ymin{"+s_Ymin+"} "+"\\def\\ymax{"+s_Ymax+"}\n";
if (afficherCadre) texcode+="\\psframe[linewidth=0.3pt,linecolor=gray]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
else texcode+="\\newrgbcolor{couleurcadre}{0.99 0.99 0.99}\n\\psframe[linewidth=0.3pt,linecolor=couleurcadre]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
texcode+="\\def\\pshlabel#1{\\psframebox*[framesep=1pt]{\\footnotesize #1}}\n";
texcode+="\\def\\psvlabel#1{\\psframebox*[framesep=1pt]{\\footnotesize #1}}\n";
if (ui.checkBoxGrille->isChecked())
	{
texcode+="\\psset{xunit="+QString::number(grilleUnitX)+"cm,yunit="+QString::number(grilleUnitY)+"cm}\n";
texcode+="\\psgrid[gridlabels=0pt,gridwidth=.3pt, gridcolor=gray, subgridwidth=.3pt, subgridcolor=gray, subgriddiv=1](0,0)("+QString::number(grilleXmin)+","+QString::number(grilleYmin)+")("+QString::number(grilleXmax)+","+QString::number(grilleYmax)+")\n";
texcode+="\\psset{xunit="+s_unitX+"cm , yunit="+s_unitY+"cm}\n";
// 	texcode+="\\multido{\\n="+QString::number(debutgrilleX)+"+"+ui.lineEditGrilleX->text()+"}{"+QString::number(nblignesX)+"}{\\psline[linewidth=0.3pt,linecolor=gray](\\n,"+ui.lineEditYmin->text()+")(\\n,"+ui.lineEditYmax->text()+")}\n";
// 	texcode+="\\multido{\\n="+QString::number(debutgrilleY)+"+"+ui.lineEditGrilleY->text()+"}{"+QString::number(nblignesY)+"}{\\psline[linewidth=0.3pt,linecolor=gray]("+ui.lineEditXmin->text()+",\\n)("+ui.lineEditXmax->text()+",\\n)}\n";
	}
QString s_Ox="0";
QString s_Oy="0";
if (Xmin>0) s_Ox="\\xmin";
if (Xmax<0) s_Ox="\\xmax";
if (Ymin>0) s_Oy="\\ymin";
if (Ymax<0) s_Oy="\\ymax";
if (ui.checkBoxAxes->isChecked())
	{
	if (ui.checkBoxLabels->isChecked()) texcode+="\\psaxes[labels=all,labelsep=1pt, Dx="+s_gradX+",Dy="+s_gradY+",Ox="+s_Ox+",Oy="+s_Oy+"]{-}("+s_Ox+","+s_Oy+")(\\xmin,\\ymin)(\\xmax,\\ymax)\n";
	else texcode+="\\psaxes[labels=none,labelsep=1pt,Dx="+s_gradX+",Dy="+s_gradY+",Ox="+s_Ox+",Oy="+s_Oy+"]{-}("+s_Ox+","+s_Oy+")(\\xmin,\\ymin)(\\xmax,\\ymax)\n";
	}
if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0))
	{
	texcode+="\\uput[dl](0,0){"+smallcode+" O}\n";
	texcode+="\\pcline[linewidth=1pt]{->}(0,0)(1,0) \\uput[d](0.5,0){"+smallcode+" $\\vec{\\imath}$}\n";
	texcode+="\\pcline[linewidth=1pt]{->}(0,0)(0,1) \\uput[l](0,0.5){"+smallcode+" $\\vec{\\jmath}$}\n";
	}
texcode+="\\psclip{%\n";
texcode+="\\psframe[linestyle=none](\\xmin,\\ymin)(\\xmax,\\ymax)\n";
texcode+="}\n";

int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
texcode+="\\newrgbcolor{couleur1}{"+QString::number(cR1 ,'g',5)+" "+QString::number(cG1 ,'g',5)+" "+QString::number(cB1 ,'g',5)+"}\n";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
texcode+="\\newrgbcolor{couleur2}{"+QString::number(cR2 ,'g',5)+" "+QString::number(cG2 ,'g',5)+" "+QString::number(cB2 ,'g',5)+"}\n";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
texcode+="\\newrgbcolor{couleur3}{"+QString::number(cR3 ,'g',5)+" "+QString::number(cG3 ,'g',5)+" "+QString::number(cB3 ,'g',5)+"}\n";

for (int i = 0; i < expressionList.count(); i++)
	{
	texcode+="\\def\\"+QString("FGHIJKLMNOPQRSTUVWXYZ").mid(i,1)+"{"+Postfixe(expressionList.at(i))+"}\n";
	if (couleurList.at(i)=="noir") couleur="black";
	else if (couleurList.at(i)=="gris") couleur="gray";
	else if (couleurList.at(i)=="gris foncé") couleur="darkgray";
	else if (couleurList.at(i)=="gris clair") couleur="lightgray";
	else if (couleurList.at(i)=="bleu") couleur="blue";
	else if (couleurList.at(i)=="rouge") couleur="red";
	else if (couleurList.at(i)=="vert") couleur="green";
	else if (couleurList.at(i)=="jaune") couleur="yellow";
	else if (couleurList.at(i)=="cyan") couleur="cyan";
	else if (couleurList.at(i)=="magenta") couleur="magenta";
	else if (couleurList.at(i)=="couleur1") couleur="couleur1";
	else if (couleurList.at(i)=="couleur2") couleur="couleur2";
	else if (couleurList.at(i)=="couleur3") couleur="couleur3";
	else couleur="black";
	if (styleList.at(i)=="ligne") style="solid";
	else if (styleList.at(i)=="trait") style="dashed";
	else if (styleList.at(i)=="point") style="dotted";
	else style="solid";
	s_varmin=CalculerExpression(varminList.at(i),"1");
	s_varmax=CalculerExpression(varmaxList.at(i),"1");
	texcode+="\\psplot[linecolor="+couleur+",linestyle="+style+",plotpoints=1000]{"+s_varmin+"}{"+s_varmax+"}{\\"+QString("FGHIJKLMNOPQRSTUVWXYZ").mid(i,1)+"}\n";
	}
texcode+=ui.textEditSupp->toPlainText()+"\n";
texcode+="\\endpsclip\n";

// QString s_Ox="0";
// QString s_Oy="0";
// if (Xmin>0) s_Ox="\\xmin";
// if (Xmax<0) s_Ox="\\xmax";
// if (Ymin>0) s_Oy="\\ymin";
// if (Ymax<0) s_Oy="\\ymax";
// if (ui.checkBoxAxes->isChecked())
// 	{
// 	if (ui.checkBoxLabels->isChecked()) texcode+="\\psaxes[labels=all,labelsep=1pt, Dx="+s_gradX+",Dy="+s_gradY+",Ox="+s_Ox+",Oy="+s_Oy+"]{-}("+s_Ox+","+s_Oy+")(\\xmin,\\ymin)(\\xmax,\\ymax)\n";
// 	else texcode+="\\psaxes[labels=none,labelsep=1pt,Dx="+s_gradX+",Dy="+s_gradY+",Ox="+s_Ox+",Oy="+s_Oy+"]{-}("+s_Ox+","+s_Oy+")(\\xmin,\\ymin)(\\xmax,\\ymax)\n";
// 	}
// if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0))
// 	{
// 	texcode+="\\uput[dl](0,0){0}\n";
// 	texcode+="\\pcline[linewidth=1pt]{->}(0,0)(1,0) \\uput[d](0.5,0){"+smallcode+" $\\vec i$}\n";
// 	texcode+="\\pcline[linewidth=1pt]{->}(0,0)(0,1) \\uput[l](0,0.5){"+smallcode+" $\\vec j$}\n";
// 	}
texcode+="\\end{pspicture*}\n";
return texcode;
}

void CourbesDialog::AfficherCode()
{
QString texcode="%\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void CourbesDialog::ajouterFonction()
{
if (expressionList.count()>19)
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de rajouter une fonction"));
	return;
	}
ActiverFonctions(true);
expressionList.append("Nouveau");
varminList.append("0");
varmaxList.append("1");
couleurList.append("noir");
styleList.append("ligne");
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetFonctions);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetFonctions->setCurrentItem(nouveauItem);
ui.listWidgetFonctions->setItemSelected(ui.listWidgetFonctions->currentItem(), true);
ui.lineEditFonction->setFocus();
NettoyerCode();
}

void CourbesDialog::supprimerFonction()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetFonctions->currentItem();
expressionList.removeAt(courant);
varminList.removeAt(courant);
varmaxList.removeAt(courant);
couleurList.removeAt(courant);
styleList.removeAt(courant);
ui.listWidgetFonctions->setCurrentItem(ui.listWidgetFonctions->item(courant));
ui.listWidgetFonctions->setItemSelected(ui.listWidgetFonctions->item(courant),true);
actualiserItemInfo();
NettoyerCode();
}

void CourbesDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetFonctions->item(courant)->setText(ui.lineEditFonction->text());
expressionList[courant]=ui.lineEditFonction->text();
ui.labelExpression->setText(QString("FGHIJKLMNOPQRSTUVWXYZ").mid(courant,1)+"(x)=");
NettoyerCode();
}

void CourbesDialog::actualiserVarmin()
{
int courant=numeroItem();
if (courant<0) return;
varminList[courant]=ui.lineEditVarmin->text();
NettoyerCode();
}

void CourbesDialog::actualiserVarmax()
{
int courant=numeroItem();
if (courant<0) return;
varmaxList[courant]=ui.lineEditVarmax->text();
NettoyerCode();
}

void CourbesDialog::actualiserCouleur()
{
int courant=numeroItem();
if (courant<0) return;
couleurList[courant]=ui.comboBoxCouleur->currentText();
NettoyerCode();
}

void CourbesDialog::actualiserStyle()
{
int courant=numeroItem();
if (courant<0) return;
styleList[courant]=ui.comboBoxStyle->currentText();
NettoyerCode();
}

void CourbesDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditFonction->setText("");
	ui.labelExpression->setText("F(x)=");
	ui.lineEditVarmin->setText("");
	ui.lineEditVarmax->setText("");
	ui.comboBoxCouleur->setCurrentIndex(0);
	ui.comboBoxStyle->setCurrentIndex(0);
	ActiverFonctions(false);
	}
else
	{
	ui.lineEditFonction->disconnect(SIGNAL(textChanged(const QString &)));
	ui.lineEditVarmin->disconnect(SIGNAL(textChanged(const QString &)));
	ui.lineEditVarmax->disconnect(SIGNAL(textChanged(const QString &)));
	ui.comboBoxCouleur->disconnect(SIGNAL(currentIndexChanged(const QString &)));
	ui.comboBoxStyle->disconnect(SIGNAL(currentIndexChanged(const QString &)));

	ui.lineEditFonction->setText(expressionList[courant]);
	ui.labelExpression->setText(QString("FGHIJKLMNOPQRSTUVWXYZ").mid(courant,1)+"(x)=");
	ui.lineEditVarmin->setText(varminList[courant]);
	ui.lineEditVarmax->setText(varmaxList[courant]);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(couleurList[courant],Qt::MatchExactly));
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(styleList[courant],Qt::MatchExactly));

	connect(ui.lineEditFonction, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	connect(ui.lineEditVarmin, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserVarmin()));
	connect(ui.lineEditVarmax, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserVarmax()));
	connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
	connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
	ActiverFonctions(true);
	}
}

int CourbesDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetFonctions->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetFonctions->setCurrentItem(items[0]);
	return ui.listWidgetFonctions->row(items[0]);
	}
return -1;
}
//******************************************************
void CourbesDialog::NouvelleCourbe()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Courbes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCourbe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Courbes : ")+nomFichier);
}

void CourbesDialog::SauverCourbe()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Courbes");
doc.appendChild(root);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("MargeX");
contenu=doc.createTextNode(ui.lineEditMargeX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("UnitY");
contenu=doc.createTextNode(ui.lineEditUniteY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(ui.lineEditYmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(ui.lineEditYmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("MargeY");
contenu=doc.createTextNode(ui.lineEditMargeY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradY");
contenu=doc.createTextNode(ui.lineEditGradY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleX");
contenu=doc.createTextNode(ui.lineEditGrilleX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleY");
contenu=doc.createTextNode(ui.lineEditGrilleY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Axes");
if (ui.checkBoxAxes->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Repere");
if (ui.checkBoxRepere->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Grille");
if (ui.checkBoxGrille->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Fonctions");
contenu=doc.createTextNode(expressionList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Varmins");
contenu=doc.createTextNode(varminList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Varmaxs");
contenu=doc.createTextNode(varmaxList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleurs");
contenu=doc.createTextNode(couleurList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Styles");
contenu=doc.createTextNode(styleList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Courbes (*.pst)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Courbes (*.pst)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Courbes : ")+nomFichier);
}

void CourbesDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Courbes")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à des courbes"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("MargeX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditMargeX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("MargeY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditMargeY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Axes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAxes->setChecked(true);
	else ui.checkBoxAxes->setChecked(false);
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Repere");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxRepere->setChecked(true);
	else ui.checkBoxRepere->setChecked(false);
	}

nodelist = doc.elementsByTagName("Grille");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxGrille->setChecked(true);
	else ui.checkBoxGrille->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Fonctions");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	expressionList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Varmins");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	varminList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Varmaxs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	varmaxList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Couleurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	couleurList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Styles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	styleList=node.toElement().text().split("#");
	}
fichier.close();
ui.listWidgetFonctions->clear();
for (int i = 0; i < expressionList.count(); i++)
	{
	if (!expressionList.at(i).isEmpty() && !varminList.at(i).isEmpty() && !varmaxList.at(i).isEmpty() && !couleurList.at(i).isEmpty() && !styleList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetFonctions);
		nouveauItem->setText(expressionList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		expressionList.removeAt(i);
		varmaxList.removeAt(i);
		varminList.removeAt(i);
		couleurList.removeAt(i);
		styleList.removeAt(i);
		}
	}
if (ui.listWidgetFonctions->count()>0)
	{
	ui.listWidgetFonctions->setCurrentItem(ui.listWidgetFonctions->item(0));
	ui.listWidgetFonctions->setItemSelected(ui.listWidgetFonctions->currentItem(), true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Courbes : ")+nomFichier);
}

void CourbesDialog::ChargerCourbe()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Courbes : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCourbe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Courbes (*.pst)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void CourbesDialog::LatexTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+texTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << "\\documentclass[10pt,a4paper]{article}\n";
out << "\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
out << "\\usepackage{amsmath}\n\\usepackage{amsfonts}\n\\usepackage{amssymb}\n";
out << "\\pagestyle{empty}\n";
out << "\\begin{document}\n";
out << "\\TeXtoEPS\n";
out << GenererCode();
out << "\\endTeXtoEPS\n";
out << "\\end{document}\n";
fichier.close();
erreurProcessus=false;
latexproc = new QProcess( this );
latexproc->setWorkingDirectory(tempDir);
connect(latexproc, SIGNAL(finished(int)),this, SLOT(finLatexProcess(int)));
latexproc->start(latexCommande+" -interaction=nonstopmode "+texTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(QString::fromUtf8("Compilation en cours..."),0,0,4,this);
progressDialog->show();
progressDialog->raise();
progressDialog->setValue(0);
}

void CourbesDialog::finLatexProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	LogDialog *logDlg = new LogDialog(this);
	logDlg->exec();
	//QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation LaTeX"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(1);
	emit latexfini();
	}
}

void CourbesDialog::DvipsTemp()
{
dvipsproc = new QProcess( this );
dvipsproc->setWorkingDirectory(tempDir);
connect(dvipsproc, SIGNAL(finished(int)),this, SLOT(finDvipsProcess(int)));
dvipsproc->start(dvipsCommande+" -Ppk -V -E -o "+psTempFile+" "+dviTempFile);
}

void CourbesDialog::finDvipsProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande dvips"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(2);
	emit dvipsfini();
	}
}

void CourbesDialog::BboxTemp()
{
bboxproc = new QProcess( this );
bboxproc->setWorkingDirectory(tempDir);
connect(bboxproc, SIGNAL(finished(int)),this, SLOT(finBboxProcess(int)));
connect(bboxproc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
bboxproc->start(gsCommande+" -q -dBATCH -dNOPAUSE -sDEVICE=bbox "+psTempFile);
}

void CourbesDialog::readInfoOutput()
{
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QString line;
QByteArray result=bboxproc->readAllStandardError();
QTextStream bbox(&result);
while ( !bbox.atEnd() ) 
	{
	line=bbox.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		int nx0=x0.toInt()-5;
		int nx1=x1.toInt()+5;
		int ny0=y0.toInt()-5;
		int ny1=y1.toInt()+5;
		QFile psf(tempDir+"/"+psTempFile);
		if (!psf.open( QIODevice::ReadOnly )) 
			{
			erreurProcessus=true;
			return;
			}
		QTextStream tpsf( &psf );
		line=tpsf.readAll();
		psf.close();
		line.replace(rxbbox,"%%BoundingBox: "+QString::number(nx0)+" "+QString::number(ny0)+" "+QString::number(nx1)+" "+QString::number(ny1));
		if (!psf.open(QIODevice::WriteOnly))
			{
			erreurProcessus=true;
			return;
			}
		QTextStream out( &psf );
		out << line;
		psf.close();
		emit bboxfini();
		}
	}
}


void CourbesDialog::finBboxProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande bbox"));
	erreurProcessus=true;
	}
else if (progressDialog) progressDialog->setValue(3);
}

void CourbesDialog::EpstopdfTemp()
{
bool ok;
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QFile psf(tempDir+"/"+psTempFile);
if (!psf.open( QIODevice::ReadOnly )) return;
QTextStream tpsf( &psf );
QString line;
while ( !tpsf.atEnd() ) 
	{
	line=tpsf.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		}
	}
psf.close();
float nx0=x0.toFloat(&ok);
float nx1=x1.toFloat(&ok);
if (nx1<=nx0) return;
float ny0=y0.toFloat(&ok);
float ny1=y1.toFloat(&ok);

epstopdfproc = new QProcess( this );
epstopdfproc->setWorkingDirectory(tempDir);
connect(epstopdfproc, SIGNAL(finished(int)),this, SLOT(finEpstopdfProcess(int)));
//epstopdfproc->start(epstopdfCommande+" "+psTempFile);
QString command=gsCommande+" -q -dBATCH -dNOPAUSE -dUseFlateCompression=false -sDEVICE=pdfwrite -sOutputFile="+pdfTempFile+" -c \"<< /PageSize ["+QString::number(nx1-nx0)+" "+QString::number(ny1-ny0)+"] >> setpagedevice\" "+QString::number(0-nx0)+" "+QString::number(0-ny0)+" translate -f "+psTempFile;
epstopdfproc->start(command);
}

void CourbesDialog::finEpstopdfProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande epstopdf"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(4);
	emit epstopdffini();
	}
}

void CourbesDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void CourbesDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Courbes ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

void CourbesDialog::NettoyerSupp()
{
ui.textEditSupp->clear();
NettoyerCode();
}

void CourbesDialog::AjouterMacro(int item)
{
ui.textEditSupp->append(codeMacros.at(item));
}

//*******************************
void CourbesDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"courbes");
aideDialog->exec();
}
//*****************************
void CourbesDialog::ExporterEps()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterEpsSuite()));
LatexTemp();
}

void CourbesDialog::ExporterEpsSuite()
{
if (!erreurProcessus)
	{
	QString epsFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Eps (*.eps)"));
	if (!epsFichier.isEmpty())
		{
		QFileInfo fi(epsFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+psTempFile,epsFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void CourbesDialog::ExporterPdf()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterPdfSuite()));
LatexTemp();
}

void CourbesDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void CourbesDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

