/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "espacedialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>

#include <math.h>

EspaceDialog::EspaceDialog(QWidget *parent,bool use_small)
    :QDialog( parent)
{
if (use_small) smallcode="\\small";
else smallcode="";
ui.setupUi(this);
#ifndef Q_WS_MACX 
setModal(true);
#endif
connect(ui.quitButton, SIGNAL(clicked()), this, SLOT(accept()));
//setWindowFlags(Qt::Window|Qt::WindowMinMaxButtonsHint);
connect(ui.genererButton, SIGNAL(clicked()), this, SLOT(AfficherCode()));
connect(ui.listWidgetObjets, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditObjet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
connect(ui.comboBoxType, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserType()));
connect(ui.comboBoxRemplissage, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserRemplissage()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterObjet()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerObjet()));
connect(ui.remonterButton, SIGNAL(clicked()), this, SLOT(remonterObjet()));
connect(ui.descendreButton, SIGNAL(clicked()), this, SLOT(descendreObjet()));


connect(ui.lineEditUnite, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditCoeff, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.spinBoxXmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxXmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxYmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxYmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxZmin, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxZmax, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));

connect(ui.pushButtonTranslater, SIGNAL(clicked()), this, SLOT(Translater()));

connect(ui.nouveauButton, SIGNAL(clicked()), this, SLOT(NouvelEspace()));
connect(ui.ouvrirButton, SIGNAL(clicked()), this, SLOT(ChargerEspace()));
connect(ui.sauverButton, SIGNAL(clicked()), this, SLOT(SauverEspace()));

connect(ui.aideButton, SIGNAL(clicked()), this, SLOT(Aide()));

connect(ui.apercuButton, SIGNAL(clicked()), this, SLOT(LatexTemp()));
connect(this, SIGNAL(latexfini()), this, SLOT(DvipsTemp()));
connect(this, SIGNAL(dvipsfini()), this, SLOT(BboxTemp()));
connect(this, SIGNAL(bboxfini()), this, SLOT(EpstopdfTemp()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));


connect(ui.epsButton, SIGNAL(clicked()), this, SLOT(ExporterEps()));
connect(ui.pdfButton, SIGNAL(clicked()), this, SLOT(ExporterPdf()));

ui.apercuButton->setShortcut(Qt::Key_F1);
ui.ouvrirButton->setShortcut(Qt::CTRL+Qt::Key_O);
ui.sauverButton->setShortcut(Qt::CTRL+Qt::Key_S);
ui.quitButton->setShortcut(Qt::CTRL+Qt::Key_Q);
ui.epsButton->setShortcut(Qt::CTRL+Qt::Key_E);
ui.pdfButton->setShortcut(Qt::CTRL+Qt::Key_P);
ui.apercuButton->setToolTip(QString::fromUtf8("F1"));
ui.ouvrirButton->setToolTip(QString::fromUtf8("CTRL+O"));
ui.sauverButton->setToolTip(QString::fromUtf8("CTRL+S"));
ui.quitButton->setToolTip(QString::fromUtf8("Quitter (CTRL+Q)"));
ui.epsButton->setToolTip(QString::fromUtf8("CTRL+E"));
ui.pdfButton->setToolTip(QString::fromUtf8("CTRL+P"));

ui.listWidgetObjets->setAlternatingRowColors(true);
ui.listWidgetObjets->setSelectionMode (QAbstractItemView::SingleSelection);

ui.comboBoxCouleur->addItems(QString("couleur1,couleur2,couleur3").split(","));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pstplus_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
texTempFile=prefixFile+".tex";
dviTempFile=prefixFile+".dvi";
psTempFile=prefixFile+".eps";
pdfTempFile=prefixFile+".pdf";
pngTempFile=prefixFile+".png";
txtTempFile=prefixFile+".txt";

highlighter = new Highlighter(ui.textEditCode->document());
highlighterSupp = new Highlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined( Q_WS_MACX )
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_WS_WIN)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
show();
}

EspaceDialog::~EspaceDialog(){
if (latexproc && latexproc->state()==QProcess::Running) 
	{
	latexproc->disconnect();
	latexproc->kill(); 
	delete latexproc ;
	}
if (dvipsproc && dvipsproc->state()==QProcess::Running) 
	{
	dvipsproc->disconnect();
	dvipsproc->kill(); 
	delete dvipsproc ;
	}
if (epstopdfproc && epstopdfproc->state()==QProcess::Running) 
	{
	epstopdfproc->disconnect();
	epstopdfproc->kill(); 
	delete epstopdfproc ;
	}
if (bboxproc && bboxproc->state()==QProcess::Running) 
	{
	bboxproc->disconnect();
	bboxproc->kill(); 
	delete bboxproc ;
	}
if (progressDialog) delete progressDialog;
}

void EspaceDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Espace : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void EspaceDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Espace : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void EspaceDialog::Init()
{
ui.textEditCode->clear();
expressionList.clear();
couleurList.clear();
styleList.clear();
typeList.clear();
remplissageList.clear();
ui.listWidgetObjets->clear();
ui.checkBoxLabels->setChecked(true);
ui.lineEditUnite->setText("1");
ui.lineEditCoeff->setText("0.5");
ui.spinBoxXmin->setValue(0);
ui.spinBoxXmax->setValue(3);
ui.spinBoxYmin->setValue(0);
ui.spinBoxYmax->setValue(3);
ui.spinBoxZmin->setValue(0);
ui.spinBoxZmax->setValue(3);

ui.textEditSupp->clear();
ActiverObjets(false);
}
//*********************************************
void EspaceDialog::ActiverObjets(bool etat)
{
ui.lineEditObjet->setEnabled(etat);
ui.comboBoxCouleur->setEnabled(etat);
ui.comboBoxStyle->setEnabled(etat);
ui.comboBoxType->setEnabled(etat);
ui.comboBoxRemplissage->setEnabled(etat);
//NettoyerCode();
}
//******************************************************************
QString EspaceDialog::GenererCode()
{
bool ok;

QString s_unitGraph=ui.lineEditUnite->text();
double unitGraph=s_unitGraph.toDouble(&ok);
if (unitGraph<=0) unitGraph=1;

QString s_coeff=ui.lineEditCoeff->text();
double coeff=s_coeff.toDouble(&ok);

int Xmin=ui.spinBoxXmin->value();
QString s_Xmin=QString::number(Xmin);

int Xmax=ui.spinBoxXmax->value();
QString s_Xmax=QString::number(Xmax);

int Ymin=ui.spinBoxYmin->value();
QString s_Ymin=QString::number(Ymin);

int Ymax=ui.spinBoxYmax->value();
QString s_Ymax=QString::number(Ymax);

int Zmin=ui.spinBoxZmin->value();
QString s_Zmin=QString::number(Zmin);

int Zmax=ui.spinBoxZmax->value();
QString s_Zmax=QString::number(Zmax);

double decal=0.5;

double x1=(double) Ymin-decal-coeff*(Xmin-decal);
double y1=(double) Zmin-decal-coeff*(Xmin-decal);

double x2=(double) Ymin-decal-coeff*(Xmax+decal);
double y2=(double) Zmin-decal-coeff*(Xmax+decal);

double cadreXmin=qMin(x1,x2);
double cadreYmin=qMin(y1,y2);
double cadreXmax=qMax(x1,x2);
double cadreYmax=qMax(y1,y2);

double x3=(double) Ymax+decal-coeff*(Xmax+decal);
double y3=(double) Zmin-decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x3);
cadreYmin=qMin(cadreYmin,y3);
cadreXmax=qMax(cadreXmax,x3);
cadreYmax=qMax(cadreYmax,y3);

double x4=(double) Ymax+decal-coeff*(Xmin-decal);
double y4=(double) Zmin-decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x4);
cadreYmin=qMin(cadreYmin,y4);
cadreXmax=qMax(cadreXmax,x4);
cadreYmax=qMax(cadreYmax,y4);

double x5=(double) Ymin-decal-coeff*(Xmin-decal);
double y5=(double) Zmax+decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x5);
cadreYmin=qMin(cadreYmin,y5);
cadreXmax=qMax(cadreXmax,x5);
cadreYmax=qMax(cadreYmax,y5);

double x6=(double) Ymin-decal-coeff*(Xmax+decal);
double y6=(double) Zmax+decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x6);
cadreYmin=qMin(cadreYmin,y6);
cadreXmax=qMax(cadreXmax,x6);
cadreYmax=qMax(cadreYmax,y6);

double x7=(double) Ymax+decal-coeff*(Xmax+decal);
double y7=(double) Zmax+decal-coeff*(Xmax+decal);

cadreXmin=qMin(cadreXmin,x7);
cadreYmin=qMin(cadreYmin,y7);
cadreXmax=qMax(cadreXmax,x7);
cadreYmax=qMax(cadreYmax,y7);

double x8=(double) Ymax+decal-coeff*(Xmin-decal);
double y8=(double) Zmax+decal-coeff*(Xmin-decal);

cadreXmin=qMin(cadreXmin,x8);
cadreYmin=qMin(cadreYmin,y8);
cadreXmax=qMax(cadreXmax,x8);
cadreYmax=qMax(cadreYmax,y8);

QString texcode="\\psset{xunit="+s_unitGraph+"cm , yunit="+s_unitGraph+"cm}\n";
texcode+="\\begin{pspicture*}("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";

// texcode+="\\psline{-}("+QString::number(x1)+","+QString::number(y1)+")("+QString::number(x2)+","+QString::number(y2)+")("+QString::number(x3)+","+QString::number(y3)+")("+QString::number(x4)+","+QString::number(y4)+")("+QString::number(x1)+","+QString::number(y1)+")\n";
// texcode+="\\psline{-}("+QString::number(x5)+","+QString::number(y5)+")("+QString::number(x6)+","+QString::number(y6)+")("+QString::number(x7)+","+QString::number(y7)+")("+QString::number(x8)+","+QString::number(y8)+")("+QString::number(x5)+","+QString::number(y5)+")\n";

if (afficherCadre) texcode+="\\psframe[linewidth=0.3pt,linecolor=gray]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";
else texcode+="\\newrgbcolor{couleurcadre}{0.99 0.99 0.99}\n\\psframe[linewidth=0.3pt,linecolor=couleurcadre]("+QString::number(cadreXmin)+","+QString::number(cadreYmin)+")("+QString::number(cadreXmax)+","+QString::number(cadreYmax)+")\n";

double xdep;
double ydep;
double xfin;
double yfin;

if (ui.checkBoxLabels->isChecked())
	{
	texcode+="\\psaxes[linewidth=1pt,showorigin=false,labels=all,labelsep=1pt, Dx=1,Dy=1]{->}(0,0)("+QString::number(Ymin)+","+QString::number(Zmin)+")("+QString::number(Ymax)+","+QString::number(Zmax)+")\n";
	texcode+="\\uput[-45](0,0){O}\n";
	xdep=-coeff;
	ydep=-coeff;
	texcode+="\\pcline[linewidth=1.75pt]{->}(0,0)("+QString::number(xdep)+","+QString::number(ydep)+")\n";
	xdep=-coeff/2;
	ydep=-coeff/2;
	texcode+="\\uput[0]("+QString::number(xdep)+","+QString::number(ydep)+"){"+smallcode+" $\\vec{\\imath}$}\n";
	texcode+="\\pcline[linewidth=1.75pt]{->}(0,0)(1,0)\n\\uput[90](0.5,0){"+smallcode+" $\\vec{\\jmath}$}\n";
	texcode+="\\pcline[linewidth=1.75pt]{->}(0,0)(0,1)\n\\uput[180](0,0.5){"+smallcode+" $\\vec{k}$}\n";
	xdep=(double) -coeff*Xmax;
	ydep=(double) -coeff*Xmax;
	xfin=(double) -coeff*Xmin;
	yfin=(double) -coeff*Xmin;
	texcode+="\\psline{<-}("+QString::number(xdep)+","+QString::number(ydep)+")("+QString::number(xfin)+","+QString::number(yfin)+")\n";
	texcode+="\\uput[-90]("+QString::number(xdep)+","+QString::number(ydep)+"){$x$}\n";
	texcode+="\\uput[0]("+QString::number(Ymax)+",0){$y$}\n";
	texcode+="\\uput[90](0,"+QString::number(Zmax)+"){$z$}\n";
	
	for (int i = Xmin; i < Xmax; i++)
		{
		if (i!=0)
			{
			xdep=(double) -coeff*i;
			texcode+="\\uput[180]("+QString::number(xdep)+","+QString::number(xdep)+"){"+QString::number(i)+"}\n";
			xdep=(double) 0.1/unitGraph-coeff*i;
			xfin=(double) -0.1/unitGraph-coeff*i;
			ydep=(double) -coeff*i;
			yfin=(double) -coeff*i;
			texcode+="\\psline{-}("+QString::number(xdep)+","+QString::number(ydep)+")("+QString::number(xfin)+","+QString::number(yfin)+")\n";
			}
		}
	}
int pR1= (couleurSupp1.red()*10000/255);
double cR1=(double) pR1/10000;
int pG1= (couleurSupp1.green()*10000/255);
double cG1=(double) pG1/10000;
int pB1= (couleurSupp1.blue()*10000/255);
double cB1=(double) pB1/10000;
texcode+="\\newrgbcolor{couleur1}{"+QString::number(cR1 ,'g',5)+" "+QString::number(cG1 ,'g',5)+" "+QString::number(cB1 ,'g',5)+"}\n";

int pR2= (couleurSupp2.red()*10000/255);
double cR2=(double) pR2/10000;
int pG2= (couleurSupp2.green()*10000/255);
double cG2=(double) pG2/10000;
int pB2= (couleurSupp2.blue()*10000/255);
double cB2=(double) pB2/10000;
texcode+="\\newrgbcolor{couleur2}{"+QString::number(cR2 ,'g',5)+" "+QString::number(cG2 ,'g',5)+" "+QString::number(cB2 ,'g',5)+"}\n";

int pR3= (couleurSupp3.red()*10000/255);
double cR3=(double) pR3/10000;
int pG3= (couleurSupp3.green()*10000/255);
double cG3=(double) pG3/10000;
int pB3= (couleurSupp3.blue()*10000/255);
double cB3=(double) pB3/10000;
texcode+="\\newrgbcolor{couleur3}{"+QString::number(cR3 ,'g',5)+" "+QString::number(cG3 ,'g',5)+" "+QString::number(cB3 ,'g',5)+"}\n";

QString couleur,style,remplissage;
QStringList val,points,sommets;
QString xpmin,ypmin,zpmin,xpmax,ypmax,zpmax;
for (int i = 0; i < expressionList.count(); i++)
	{
	if (couleurList.at(i)=="noir") couleur="black";
	else if (couleurList.at(i)=="gris") couleur="gray";
	else if (couleurList.at(i)=="gris foncé") couleur="darkgray";
	else if (couleurList.at(i)=="gris clair") couleur="lightgray";
	else if (couleurList.at(i)=="bleu") couleur="blue";
	else if (couleurList.at(i)=="rouge") couleur="red";
	else if (couleurList.at(i)=="vert") couleur="green";
	else if (couleurList.at(i)=="jaune") couleur="yellow";
	else if (couleurList.at(i)=="cyan") couleur="cyan";
	else if (couleurList.at(i)=="magenta") couleur="magenta";
	else if (couleurList.at(i)=="couleur1") couleur="couleur1";
	else if (couleurList.at(i)=="couleur2") couleur="couleur2";
	else if (couleurList.at(i)=="couleur3") couleur="couleur3";
	else if (couleurList.at(i)=="blanc") couleur="white";
	else couleur="black";
	if (styleList.at(i)=="ligne") style="solid";
	else if (styleList.at(i)=="trait") style="dashed";
	else if (styleList.at(i)=="point") style="dotted";
	else style="solid";
	if (remplissageList.at(i)=="plein") remplissage="fillcolor="+couleur+",fillstyle=solid";
	else if (remplissageList.at(i)=="lignes") remplissage="fillcolor="+couleur+",fillstyle=hlines,hatchwidth=0.3pt,hatchsep=12pt,hatchcolor="+couleur;
	else if (remplissageList.at(i)=="hachures") remplissage="fillcolor="+couleur+",fillstyle=crosshatch,hatchwidth=0.3pt,hatchsep=12pt,hatchcolor="+couleur;
	else remplissage="";
	if (typeList.at(i)=="Point")
		{
		val=expressionList.at(i).split(";");
		if (val.count()==3)
			{
			xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
			ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
			texcode+="\\psdot[linecolor="+couleur+"]("+QString::number(xdep)+","+QString::number(ydep)+")\n";
			}
		}
	else if (typeList.at(i)=="Point avec traces")
		{
		val=expressionList.at(i).split(";");
		if (val.count()==3)
			{
			xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
			ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
			texcode+="\\psdot[linecolor="+couleur+"]("+QString::number(xdep)+","+QString::number(ydep)+")\n";
			x1=(double) -coeff*val.at(0).toDouble();
			y1=(double) -coeff*val.at(0).toDouble();
			x2=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
			y2=(double) -coeff*val.at(0).toDouble();
			x3=val.at(1).toDouble();
			y3=0;
			x4=0;
			y4=val.at(2).toDouble();
			texcode+="\\psline[linecolor=darkgray,linewidth=0.5pt,linestyle=dashed]("+QString::number(x1)+","+QString::number(y1)+")("+QString::number(x2)+","+QString::number(y2)+")("+QString::number(x3)+","+QString::number(y3)+")\n";
			texcode+="\\psline[linecolor=darkgray,linewidth=0.5pt,linestyle=dashed](0,0)("+QString::number(x2)+","+QString::number(y2)+")("+QString::number(xdep)+","+QString::number(ydep)+")("+QString::number(x4)+","+QString::number(y4)+")\n";
			}
		}
	else if (typeList.at(i)=="Segment")
		{
		points=expressionList.at(i).split(":");
		if (points.count()==2)
			{
			texcode+="\\psline[linewidth=1pt,linecolor="+couleur+",linestyle="+style+"]{-}";
			for (int j = 0; j < points.count(); j++)
				{
				val=points.at(j).split(";");
				xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
				ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
				texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
				}
			texcode+="\n";
			}
		}
	else if (typeList.at(i)=="Vecteur")
		{
		points=expressionList.at(i).split(":");
		if (points.count()==2)
			{
			texcode+="\\psline[linewidth=1pt,arrowsize=4pt,linecolor="+couleur+",linestyle="+style+"]{->}";
			for (int j = 0; j < points.count(); j++)
				{
				val=points.at(j).split(";");
				xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
				ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
				texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
				}
			texcode+="\n";
			}
		}
	else if (typeList.at(i)=="Polygone")
		{
		points=expressionList.at(i).split(":");
		if (points.count()>0)
			{
			texcode+="\\psline[linewidth=1pt,linecolor="+couleur+",linestyle="+style+","+remplissage+"]{-}";
			for (int j = 0; j < points.count(); j++)
				{
				val=points.at(j).split(";");
				xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
				ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
				texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
				}
			texcode+="\n";
			}
		}
	else if (typeList.at(i)=="Label")
		{
		val=expressionList.at(i).split(";");
		if (val.count()==5)
			{
			xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
			ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
			texcode+="\\uput["+val.at(3)+"]("+QString::number(xdep)+","+QString::number(ydep)+"){"+val.at(4)+"}\n";
			}
		}
	else if (typeList.at(i)==QString::fromUtf8("Pavé droit"))
		{
		points=expressionList.at(i).split(":");
		if (points.count()==2)
			{
			val=points.at(0).split(";");
			if (val.count()==3)
				{
				xpmin=val.at(0);
				ypmin=val.at(1);
				zpmin=val.at(2);
				val=points.at(1).split(";");
				if (val.count()==3)
					{
					xpmax=val.at(0);
					ypmax=val.at(1);
					zpmax=val.at(2);
					sommets.clear();
					sommets.append(xpmin+";"+ypmin+";"+zpmax);
					sommets.append(xpmax+";"+ypmin+";"+zpmax);
					sommets.append(xpmax+";"+ypmax+";"+zpmax);
					sommets.append(xpmin+";"+ypmax+";"+zpmax);
					sommets.append(xpmin+";"+ypmin+";"+zpmax);
					sommets.append(xpmax+";"+ypmin+";"+zpmax);
					sommets.append(xpmax+";"+ypmin+";"+zpmin);
					sommets.append(xpmax+";"+ypmax+";"+zpmin);
					sommets.append(xpmax+";"+ypmax+";"+zpmax);
					sommets.append(xpmin+";"+ypmax+";"+zpmax);
					sommets.append(xpmin+";"+ypmax+";"+zpmin);
					sommets.append(xpmax+";"+ypmax+";"+zpmin);
					if (sommets.count()>0)
						{
						texcode+="\\psline[linewidth=1pt,linecolor="+couleur+",linestyle="+style+"]{-}";
						for (int j = 0; j < sommets.count(); j++)
							{
							val=sommets.at(j).split(";");
							xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
							ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
							texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
							}
						texcode+="\n";
						}
					sommets.clear();
					sommets.append(xpmax+";"+ypmin+";"+zpmin);
					sommets.append(xpmin+";"+ypmin+";"+zpmin);
					sommets.append(xpmin+";"+ypmax+";"+zpmin);
					if (sommets.count()>0)
						{
						texcode+="\\psline[linewidth=1pt,linecolor="+couleur+",linestyle=dashed]{-}";
						for (int j = 0; j < sommets.count(); j++)
							{
							val=sommets.at(j).split(";");
							xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
							ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
							texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
							}
						texcode+="\n";
						}
					sommets.clear();
					sommets.append(xpmin+";"+ypmin+";"+zpmin);
					sommets.append(xpmin+";"+ypmin+";"+zpmax);
					if (sommets.count()>0)
						{
						texcode+="\\psline[linewidth=1pt,linecolor="+couleur+",linestyle=dashed]{-}";
						for (int j = 0; j < sommets.count(); j++)
							{
							val=sommets.at(j).split(";");
							xdep=(double) val.at(1).toDouble()-coeff*val.at(0).toDouble();
							ydep=(double) val.at(2).toDouble()-coeff*val.at(0).toDouble();
							texcode+="("+QString::number(xdep)+","+QString::number(ydep)+")";
							}
						texcode+="\n";
						}
					}
				}
			}
		}
	}

texcode+=ui.textEditSupp->toPlainText()+"\n";

texcode+="\\end{pspicture*}\n";
return texcode;
}

void EspaceDialog::AfficherCode()
{
QString texcode="%\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void EspaceDialog::ajouterObjet()
{
ActiverObjets(true);
expressionList.append("Nouveau");
couleurList.append(ui.comboBoxCouleur->currentText());
styleList.append(ui.comboBoxStyle->currentText());
typeList.append(ui.comboBoxType->currentText());
remplissageList.append(ui.comboBoxRemplissage->currentText());
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetObjets);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetObjets->setCurrentItem(nouveauItem);
ui.listWidgetObjets->setItemSelected(ui.listWidgetObjets->currentItem(), true);
ui.lineEditObjet->setFocus();
NettoyerCode();
}

void EspaceDialog::supprimerObjet()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetObjets->currentItem();
expressionList.removeAt(courant);
couleurList.removeAt(courant);
styleList.removeAt(courant);
typeList.removeAt(courant);
remplissageList.removeAt(courant);
ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant));
ui.listWidgetObjets->setItemSelected(ui.listWidgetObjets->item(courant),true);
actualiserItemInfo();
NettoyerCode();
}

void EspaceDialog::remonterObjet()
{
int courant=numeroItem();
if (courant<=0) return;
QListWidgetItem *item=ui.listWidgetObjets->item(courant)->clone();
delete ui.listWidgetObjets->item(courant);
ui.listWidgetObjets->insertItem(courant-1,item);
expressionList.swap(courant,courant-1);
couleurList.swap(courant,courant-1);
styleList.swap(courant,courant-1);
typeList.swap(courant,courant-1);
remplissageList.swap(courant,courant-1);
ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant-1));
ui.listWidgetObjets->setItemSelected(ui.listWidgetObjets->item(courant-1),true);
NettoyerCode();
}

void EspaceDialog::descendreObjet()
{
int courant=numeroItem();
if (courant>=ui.listWidgetObjets->count()-1 || courant<0) return;
QListWidgetItem *item=ui.listWidgetObjets->item(courant)->clone();
delete ui.listWidgetObjets->item(courant);
ui.listWidgetObjets->insertItem(courant+1,item);
expressionList.swap(courant,courant+1);
couleurList.swap(courant,courant+1);
styleList.swap(courant,courant+1);
typeList.swap(courant,courant+1);
remplissageList.swap(courant,courant+1);
ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(courant+1));
ui.listWidgetObjets->setItemSelected(ui.listWidgetObjets->item(courant+1),true);
NettoyerCode();
}

void EspaceDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetObjets->item(courant)->setText(ui.lineEditObjet->text());
expressionList[courant]=ui.lineEditObjet->text();
NettoyerCode();
}

void EspaceDialog::actualiserCouleur()
{
int courant=numeroItem();
if (courant<0) return;
couleurList[courant]=ui.comboBoxCouleur->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserStyle()
{
int courant=numeroItem();
if (courant<0) return;
styleList[courant]=ui.comboBoxStyle->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserType()
{
int courant=numeroItem();
if (courant<0) return;
typeList[courant]=ui.comboBoxType->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserRemplissage()
{
int courant=numeroItem();
if (courant<0) return;
remplissageList[courant]=ui.comboBoxRemplissage->currentText();
NettoyerCode();
}

void EspaceDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditObjet->setText("");
	ui.comboBoxCouleur->setCurrentIndex(0);
	ui.comboBoxStyle->setCurrentIndex(0);
	ui.comboBoxType->setCurrentIndex(0);
	ui.comboBoxRemplissage->setCurrentIndex(0);
	ActiverObjets(false);
	}
else
	{
	ui.lineEditObjet->disconnect(SIGNAL(textChanged(const QString &)));
	ui.comboBoxCouleur->disconnect(SIGNAL(currentIndexChanged(const QString &)));
	ui.comboBoxStyle->disconnect(SIGNAL(currentIndexChanged(const QString &)));
	ui.comboBoxType->disconnect(SIGNAL(currentIndexChanged(const QString &)));
	ui.comboBoxRemplissage->disconnect(SIGNAL(currentIndexChanged(const QString &)));

	ui.lineEditObjet->setText(expressionList[courant]);
	ui.comboBoxCouleur->setCurrentIndex(ui.comboBoxCouleur->findText(couleurList[courant],Qt::MatchExactly));
	ui.comboBoxStyle->setCurrentIndex(ui.comboBoxStyle->findText(styleList[courant],Qt::MatchExactly));
	ui.comboBoxType->setCurrentIndex(ui.comboBoxType->findText(typeList[courant],Qt::MatchExactly));
	ui.comboBoxRemplissage->setCurrentIndex(ui.comboBoxRemplissage->findText(remplissageList[courant],Qt::MatchExactly));

	connect(ui.lineEditObjet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	connect(ui.comboBoxCouleur, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserCouleur()));
	connect(ui.comboBoxStyle, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserStyle()));
	connect(ui.comboBoxType, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserType()));
	connect(ui.comboBoxRemplissage, SIGNAL(currentIndexChanged(const QString &)),this,SLOT(actualiserRemplissage()));
	ActiverObjets(true);
	}
}

int EspaceDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetObjets->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetObjets->setCurrentItem(items[0]);
	return ui.listWidgetObjets->row(items[0]);
	}
return -1;
}

void EspaceDialog::Translater()
{
bool ok;
int courant=numeroItem();
if (courant<0) return;
QStringList val,points;
QString nouveaucode="";

QString sx=ui.lineEditTranslateX->text();
double tx=sx.toDouble(&ok);
if (!ok) return;
QString sy=ui.lineEditTranslateY->text();
double ty=sy.toDouble(&ok);
if (!ok) return;
QString sz=ui.lineEditTranslateZ->text();
double tz=sz.toDouble(&ok);
if (!ok) return;

points=expressionList.at(courant).split(":");
if (points.count()<=0) return;
for (int i = 0; i < points.count(); i++)
	{
	val=points.at(i).split(";");
	if (val.count()>=3)
		{
		double xorig=(double) val.at(0).toDouble();
		double nouvx=xorig+tx;
		double yorig=(double) val.at(1).toDouble();
		double nouvy=yorig+ty;
		double zorig=(double) val.at(2).toDouble();
		double nouvz=zorig+tz;
		nouveaucode+=QString::number(nouvx)+";"+QString::number(nouvy)+";"+QString::number(nouvz);
		if (val.count()>3)
			{		
			for (int j = 3; j < val.count(); j++) nouveaucode+=";"+val.at(j);
			}
		}
	if (i<points.count()-1) nouveaucode+=":";
	}
expressionList[courant]=nouveaucode;
ui.lineEditObjet->setText(nouveaucode);
ui.lineEditTranslateX->setText("0");
ui.lineEditTranslateY->setText("0");
ui.lineEditTranslateZ->setText("0");
}
//******************************************************
void EspaceDialog::NouvelEspace()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Espace : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::SauverEspace()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Espace");
doc.appendChild(root);

description=doc.createElement("Unit");
contenu=doc.createTextNode(ui.lineEditUnite->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Coeff");
contenu=doc.createTextNode(ui.lineEditCoeff->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(QString::number(ui.spinBoxXmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(QString::number(ui.spinBoxXmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(QString::number(ui.spinBoxYmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(QString::number(ui.spinBoxYmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmin");
contenu=doc.createTextNode(QString::number(ui.spinBoxZmin->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmax");
contenu=doc.createTextNode(QString::number(ui.spinBoxZmax->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Objets");
contenu=doc.createTextNode(expressionList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Couleurs");
contenu=doc.createTextNode(couleurList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Styles");
contenu=doc.createTextNode(styleList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Types");
contenu=doc.createTextNode(typeList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Remplissages");
contenu=doc.createTextNode(remplissageList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Espace (*.pst)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Espace (*.pst)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Espace")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un repère dans l'espace"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Unit");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUnite->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Coeff");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditCoeff->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxXmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxXmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxYmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxYmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Zmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxZmin->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Zmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxZmax->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Objets");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	expressionList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Couleurs");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	couleurList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Styles");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	styleList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Types");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	typeList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Remplissages");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	remplissageList=node.toElement().text().split("#");
	}

fichier.close();
ui.listWidgetObjets->clear();
for (int i = 0; i < expressionList.count(); i++)
	{
	if (!expressionList.at(i).isEmpty() && !couleurList.at(i).isEmpty() && !styleList.at(i).isEmpty() && !typeList.at(i).isEmpty() && !remplissageList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetObjets);
		nouveauItem->setText(expressionList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		expressionList.removeAt(i);
		couleurList.removeAt(i);
		styleList.removeAt(i);
		typeList.removeAt(i);
		remplissageList.removeAt(i);
		}
	}
if (ui.listWidgetObjets->count()>0)
	{
	ui.listWidgetObjets->setCurrentItem(ui.listWidgetObjets->item(0));
	ui.listWidgetObjets->setItemSelected(ui.listWidgetObjets->currentItem(), true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Espace : ")+nomFichier);
}

void EspaceDialog::ChargerEspace()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Espace : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverEspace();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Espace (*.pst)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void EspaceDialog::LatexTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+texTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << "\\documentclass[10pt,a4paper]{article}\n";
out << "\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
out << "\\usepackage{amsmath}\n\\usepackage{amsfonts}\n\\usepackage{amssymb}\n";
out << "\\pagestyle{empty}\n";
out << "\\begin{document}\n";
out << "\\TeXtoEPS\n";
out << GenererCode();
out << "\\endTeXtoEPS\n";
out << "\\end{document}\n";
fichier.close();
erreurProcessus=false;
latexproc = new QProcess( this );
latexproc->setWorkingDirectory(tempDir);
connect(latexproc, SIGNAL(finished(int)),this, SLOT(finLatexProcess(int)));
latexproc->start(latexCommande+" -interaction=nonstopmode "+texTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(QString::fromUtf8("Compilation en cours..."),0,0,4,this);
progressDialog->show();
progressDialog->raise();
progressDialog->setValue(0);
}

void EspaceDialog::finLatexProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	LogDialog *logDlg = new LogDialog(this);
	logDlg->exec();
	//QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation LaTeX"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(1);
	emit latexfini();
	}
}

void EspaceDialog::DvipsTemp()
{
dvipsproc = new QProcess( this );
dvipsproc->setWorkingDirectory(tempDir);
connect(dvipsproc, SIGNAL(finished(int)),this, SLOT(finDvipsProcess(int)));
dvipsproc->start(dvipsCommande+" -Ppk -V -E -o "+psTempFile+" "+dviTempFile);
}

void EspaceDialog::finDvipsProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande dvips"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(2);
	emit dvipsfini();
	}
}

void EspaceDialog::BboxTemp()
{
bboxproc = new QProcess( this );
bboxproc->setWorkingDirectory(tempDir);
connect(bboxproc, SIGNAL(finished(int)),this, SLOT(finBboxProcess(int)));
connect(bboxproc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
bboxproc->start(gsCommande+" -q -dBATCH -dNOPAUSE -sDEVICE=bbox "+psTempFile);
}

void EspaceDialog::readInfoOutput()
{
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QString line;
QByteArray result=bboxproc->readAllStandardError();
QTextStream bbox(&result);
while ( !bbox.atEnd() ) 
	{
	line=bbox.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		int nx0=x0.toInt()-5;
		int nx1=x1.toInt()+5;
		int ny0=y0.toInt()-5;
		int ny1=y1.toInt()+5;
		QFile psf(tempDir+"/"+psTempFile);
		if (!psf.open( QIODevice::ReadOnly )) 
			{
			erreurProcessus=true;
			return;
			}
		QTextStream tpsf( &psf );
		line=tpsf.readAll();
		psf.close();
		line.replace(rxbbox,"%%BoundingBox: "+QString::number(nx0)+" "+QString::number(ny0)+" "+QString::number(nx1)+" "+QString::number(ny1));
		if (!psf.open(QIODevice::WriteOnly))
			{
			erreurProcessus=true;
			return;
			}
		QTextStream out( &psf );
		out << line;
		psf.close();
		emit bboxfini();
		}
	}
}


void EspaceDialog::finBboxProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande bbox"));
	erreurProcessus=true;
	}
else if (progressDialog) progressDialog->setValue(3);
}

void EspaceDialog::EpstopdfTemp()
{
bool ok;
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QFile psf(tempDir+"/"+psTempFile);
if (!psf.open( QIODevice::ReadOnly )) return;
QTextStream tpsf( &psf );
QString line;
while ( !tpsf.atEnd() ) 
	{
	line=tpsf.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		}
	}
psf.close();
float nx0=x0.toFloat(&ok);
float nx1=x1.toFloat(&ok);
if (nx1<=nx0) return;
float ny0=y0.toFloat(&ok);
float ny1=y1.toFloat(&ok);

epstopdfproc = new QProcess( this );
epstopdfproc->setWorkingDirectory(tempDir);
connect(epstopdfproc, SIGNAL(finished(int)),this, SLOT(finEpstopdfProcess(int)));
//epstopdfproc->start(epstopdfCommande+" "+psTempFile);
QString command=gsCommande+" -q -dBATCH -dNOPAUSE -dUseFlateCompression=false -sDEVICE=pdfwrite -sOutputFile="+pdfTempFile+" -c \"<< /PageSize ["+QString::number(nx1-nx0)+" "+QString::number(ny1-ny0)+"] >> setpagedevice\" "+QString::number(0-nx0)+" "+QString::number(0-ny0)+" translate -f "+psTempFile;
epstopdfproc->start(command);
}

void EspaceDialog::finEpstopdfProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande epstopdf"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(4);
	emit epstopdffini();
	}
}


void EspaceDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void EspaceDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Espace ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void EspaceDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"espace");
aideDialog->exec();
}
//*****************************
void EspaceDialog::ExporterEps()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterEpsSuite()));
LatexTemp();
}

void EspaceDialog::ExporterEpsSuite()
{
if (!erreurProcessus)
	{
	QString epsFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Eps (*.eps)"));
	if (!epsFichier.isEmpty())
		{
		QFileInfo fi(epsFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+psTempFile,epsFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void EspaceDialog::ExporterPdf()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterPdfSuite()));
LatexTemp();
}

void EspaceDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void EspaceDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

