/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QtGui>

#include "highlighter.h"

Highlighter::Highlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
    HighlightingRule rule;

    psexpressionFormat.setForeground(QColor(0x00, 0x00, 0xCC));
    rule.pattern = QRegExp("(\\\\ps[A-Za-z]+|\\\\uput|\\\\def|\\\\pcline|\\\\ncline|\\\\ncarc|\\\\nccircle)");
    rule.format = psexpressionFormat;
    highlightingRules.append(rule);

    figureFormat.setFontWeight(QFont::Bold);
    figureFormat.setForeground(QColor(0x80, 0x00, 0x00));
    rule.pattern = QRegExp("(\\\\begin\\{pspicture\\*\\}|\\\\end\\{pspicture\\*\\}|\\\\psclip|\\\\endpsclip|\\\\begin\\{psmatrix\\}|\\\\end\\{psmatrix\\})");
    rule.format = figureFormat;
    highlightingRules.append(rule);

    erreurFormat.setFontWeight(QFont::Bold);
    erreurFormat.setForeground(Qt::red);
    rule.pattern = QRegExp("(Erreur|erreur)");
    rule.format = erreurFormat;
    highlightingRules.append(rule);

    commentaireFormat.setFontItalic(true);
    commentaireFormat.setForeground(QColor(0x83, 0x81, 0x83));
    rule.pattern = QRegExp("%[^\n]*");
    rule.format = commentaireFormat;
    highlightingRules.append(rule);

}

void Highlighter::highlightBlock(const QString &text)
{
foreach (HighlightingRule rule, highlightingRules) 
	{
	QRegExp expression(rule.pattern);
	int index = text.indexOf(expression);
	while (index >= 0) 
		{
		int length = expression.matchedLength();
		setFormat(index, length, rule.format);
		index = text.indexOf(expression, index + length);
		}
	}
}
