/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "surfacedialog.h"
#include "aidedialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>

#include <math.h>

SurfaceDialog::SurfaceDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#ifndef Q_WS_MACX 
setModal(true);
#endif
//setWindowFlags(Qt::Window|Qt::WindowMinMaxButtonsHint);
connect(ui.quitButton, SIGNAL(clicked()), this, SLOT(accept()));
connect(ui.genererButton, SIGNAL(clicked()), this, SLOT(AfficherCode()));

connect(ui.lineEditFonction, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditIsoX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditIsoY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditZmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditZmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradZ, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditIsoZ, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.spinBoxRotX, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.spinBoxRotZ, SIGNAL(valueChanged(int)),this, SLOT(NettoyerCode()));
connect(ui.lineEditEchelle, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditEchelleZ, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.listWidgetPoints, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditPoints, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterPoint()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerPoint()));

connect(ui.nouveauButton, SIGNAL(clicked()), this, SLOT(NouvelleSurface()));
connect(ui.ouvrirButton, SIGNAL(clicked()), this, SLOT(ChargerSurface()));
connect(ui.sauverButton, SIGNAL(clicked()), this, SLOT(SauverSurface()));

connect(ui.aideButton, SIGNAL(clicked()), this, SLOT(Aide()));

connect(ui.apercuButton, SIGNAL(clicked()), this, SLOT(GnuplotTemp()));
connect(this, SIGNAL(gnuplotfini()), this, SLOT(BboxTemp()));
connect(this, SIGNAL(bboxfini()), this, SLOT(EpstopdfTemp()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));


connect(ui.epsButton, SIGNAL(clicked()), this, SLOT(ExporterEps()));
connect(ui.pdfButton, SIGNAL(clicked()), this, SLOT(ExporterPdf()));

ui.apercuButton->setShortcut(Qt::Key_F1);
ui.ouvrirButton->setShortcut(Qt::CTRL+Qt::Key_O);
ui.sauverButton->setShortcut(Qt::CTRL+Qt::Key_S);
ui.quitButton->setShortcut(Qt::CTRL+Qt::Key_Q);
ui.epsButton->setShortcut(Qt::CTRL+Qt::Key_E);
ui.pdfButton->setShortcut(Qt::CTRL+Qt::Key_P);
ui.apercuButton->setToolTip(QString::fromUtf8("F1"));
ui.ouvrirButton->setToolTip(QString::fromUtf8("CTRL+O"));
ui.sauverButton->setToolTip(QString::fromUtf8("CTRL+S"));
ui.quitButton->setToolTip(QString::fromUtf8("Quitter (CTRL+Q)"));
ui.epsButton->setToolTip(QString::fromUtf8("CTRL+E"));
ui.pdfButton->setToolTip(QString::fromUtf8("CTRL+P"));

ui.listWidgetPoints->setAlternatingRowColors(true);
ui.listWidgetPoints->setSelectionMode (QAbstractItemView::SingleSelection);

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pstplus_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
gnuplotTempFile=prefixFile+".gpl";
psTempFile=prefixFile+".eps";
pdfTempFile=prefixFile+".pdf";
pngTempFile=prefixFile+".png";
txtTempFile=prefixFile+".txt";


Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Surface 3D : ")+nomFichier);
show();
}

SurfaceDialog::~SurfaceDialog(){
if (gnuplotproc && gnuplotproc->state()==QProcess::Running) 
	{
	gnuplotproc->disconnect();
	gnuplotproc->kill(); 
	delete gnuplotproc ;
	}
if (voirproc && voirproc->state()==QProcess::Running) 
	{
	voirproc->kill(); 
	delete voirproc ;
	}
if (epstopdfproc && epstopdfproc->state()==QProcess::Running) 
	{
	epstopdfproc->disconnect();
	epstopdfproc->kill(); 
	delete epstopdfproc ;
	}
if (bboxproc && bboxproc->state()==QProcess::Running) 
	{
	bboxproc->disconnect();
	bboxproc->kill(); 
	delete bboxproc ;
	}
if (progressDialog) delete progressDialog;
}

void SurfaceDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Surface : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSurface();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void SurfaceDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Surface : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSurface();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void SurfaceDialog::Init()
{
ui.textEditCode->clear();
pointList.clear();
ui.listWidgetPoints->clear();
ui.lineEditXmin->setText("0");
ui.lineEditXmax->setText("1");
ui.lineEditGradX->setText("1");
ui.lineEditIsoX->setText("1");
ui.lineEditYmin->setText("0");
ui.lineEditYmax->setText("1");
ui.lineEditGradY->setText("1");
ui.lineEditIsoY->setText("1");
ui.lineEditZmin->setText("0");
ui.lineEditZmax->setText("1");
ui.lineEditGradZ->setText("1");
ui.lineEditIsoZ->setText("1");
ui.spinBoxRotX->setValue(60);
ui.spinBoxRotZ->setValue(320);
ui.lineEditEchelle->setText("1");
ui.lineEditEchelleZ->setText("1");
ui.lineEditFonction->setText("");

ActiverPoints(false);
}
//*********************************************
void SurfaceDialog::ActiverPoints(bool etat)
{
ui.lineEditPoints->setEnabled(etat);
//NettoyerCode();
}
//******************************************************************
QString SurfaceDialog::GenererCode()
{
QStringList val;
QString texcode="set encoding iso_8859_1\nset term postscript eps enhanced monochrom\n";
texcode+="set angles radians\n";
texcode+="set output '"+psTempFile+"'\n";
texcode+="set border  116 #lt -1 lw 1.000\n";
texcode+="set size square 1.2,1.2\n";
texcode+="f(x,y)="+ui.lineEditFonction->text().replace("^","**")+"\n";
texcode+="set xrange["+ui.lineEditXmin->text()+":"+ui.lineEditXmax->text()+"]\n";
texcode+="set yrange["+ui.lineEditYmin->text()+":"+ui.lineEditYmax->text()+"]\n";
texcode+="set zrange["+ui.lineEditZmin->text()+":"+ui.lineEditZmax->text()+"]\n";
texcode+="set xtics "+ui.lineEditXmin->text()+","+ui.lineEditGradX->text()+","+ui.lineEditXmax->text()+"\n";
texcode+="set ytics "+ui.lineEditYmin->text()+","+ui.lineEditGradY->text()+","+ui.lineEditYmax->text()+"\n";
texcode+="set ztics "+ui.lineEditZmin->text()+","+ui.lineEditGradZ->text()+","+ui.lineEditZmax->text()+"\n";
texcode+="set xlabel \"X\"\n";
texcode+="set ylabel \"Y\"\n";
texcode+="set zlabel \"Z\"\n";
for (int i = 0; i < pointList.count(); i++)
	{
	val=pointList.at(i).split(";");
	if (val.count()==5)
		{
		texcode+="set label \""+val.at(3)+"\" at "+val.at(0)+"+"+val.at(4)+","+val.at(1)+"+"+val.at(4)+","+val.at(2)+"+"+val.at(4)+" front\n";
		}
	}
texcode+="unset key\n";
texcode+="set ticslevel 0\n";
texcode+="set view "+QString::number(ui.spinBoxRotX->value())+","+QString::number(ui.spinBoxRotZ->value())+","+ui.lineEditEchelle->text()+","+ui.lineEditEchelleZ->text()+"\n";

//texcode+="set samples 500, 500\n";
double xmax=ui.lineEditXmax->text().toDouble();
double xmin=ui.lineEditXmin->text().toDouble();
double isox=ui.lineEditIsoX->text().toDouble();
double samplex=1;
if (isox!=0) samplex=(xmax-xmin)/isox+1;
double ymax=ui.lineEditYmax->text().toDouble();
double ymin=ui.lineEditYmin->text().toDouble();
double isoy=ui.lineEditIsoY->text().toDouble();
double sampley=1;
if (isoy!=0) sampley=(ymax-ymin)/isoy+1;
double zmax=ui.lineEditZmax->text().toDouble();
double zmin=ui.lineEditZmin->text().toDouble();
double isoz=ui.lineEditIsoZ->text().toDouble();
int samplez=1;
if (isoz!=0) samplez=(int) (zmax-zmin)/isoz;

texcode+="set grid  xtics lt 1 lw 1\n";
texcode+="set grid  ytics lt 1 lw 1\n";
texcode+="set grid  ztics lt 1 lw 1\n";
texcode+="set contour surface\n";
texcode+="set palette gray negative\n";
texcode+="#Pour avoir une couleur par bande, utiliser :\n";
texcode+="#set palette maxcolors "+QString::number(samplez)+"\n";
texcode+="set colorbox vert user origin .02,.4 size .04,.6\n";
texcode+="set noclabel\n";
texcode+="set cbrange["+ui.lineEditZmin->text()+":"+ui.lineEditZmax->text()+"]\n";
texcode+="set cbtics "+ui.lineEditZmin->text()+","+ui.lineEditGradZ->text()+","+ui.lineEditZmax->text()+"\n";

texcode+="set multiplot\n";
texcode+="#Pour avoir une couleur par bande, augmenter les isosamples\n";
texcode+="#(attention, cela augmente fortement la taille de l'image)\n";
texcode+="set isosamples "+QString::number(10*samplex)+", "+QString::number(10*sampley)+"\n";
texcode+="set samples "+QString::number(10*samplex)+", "+QString::number(10*sampley)+"\n";
texcode+="set cntrparam levels incremental "+ui.lineEditZmin->text()+","+ui.lineEditIsoZ->text()+","+ui.lineEditZmax->text()+"\n";
texcode+="splot f(x,y) with pm3d at s\n";
texcode+="unset pm3d\n";
texcode+="set hidden3d\n";
texcode+="set isosamples "+QString::number(samplex)+", "+QString::number(sampley)+"\n";
texcode+="set cntrparam levels incremental "+ui.lineEditZmin->text()+","+ui.lineEditIsoZ->text()+","+ui.lineEditZmax->text()+"\n";
if (pointList.isEmpty()) texcode+="splot  f(x,y) lt 1 lw 2\n";
else
	{
	texcode+="splot  f(x,y) lt 1 lw 2 ,\"-\" notitle with points pt 7 ps 1\n";
	for (int i = 0; i < pointList.count(); i++)
		{
		val=pointList.at(i).split(";");
		if (val.count()>=3)
			{
			texcode+="       "+val.at(0)+" "+val.at(1)+" "+val.at(2)+" \n";
			}
		}
	}
texcode+="unset multiplot\n";
return texcode;
}

void SurfaceDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void SurfaceDialog::ajouterPoint()
{
ActiverPoints(true);
pointList.append("Nouveau");
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetPoints);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidgetPoints->setCurrentItem(nouveauItem);
ui.listWidgetPoints->setItemSelected(ui.listWidgetPoints->currentItem(), true);
ui.lineEditPoints->setFocus();
NettoyerCode();
}

void SurfaceDialog::supprimerPoint()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetPoints->currentItem();
pointList.removeAt(courant);
ui.listWidgetPoints->setCurrentItem(ui.listWidgetPoints->item(courant));
ui.listWidgetPoints->setItemSelected(ui.listWidgetPoints->item(courant),true);
actualiserItemInfo();
NettoyerCode();
}

void SurfaceDialog::actualiserExpression()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetPoints->item(courant)->setText(ui.lineEditPoints->text());
pointList[courant]=ui.lineEditPoints->text();
NettoyerCode();
}

void SurfaceDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditPoints->setText("");
	ActiverPoints(false);
	}
else
	{
	ui.lineEditPoints->disconnect(SIGNAL(textChanged(const QString &)));
	ui.lineEditPoints->setText(pointList[courant]);
	connect(ui.lineEditPoints, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserExpression()));
	ActiverPoints(true);
	}
}

int SurfaceDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetPoints->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetPoints->setCurrentItem(items[0]);
	return ui.listWidgetPoints->row(items[0]);
	}
return -1;
}
//******************************************************
void SurfaceDialog::NouvelleSurface()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Surface : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSurface();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Surface : ")+nomFichier);
}

void SurfaceDialog::SauverSurface()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Surface");
doc.appendChild(root);

description=doc.createElement("Fonction");
contenu=doc.createTextNode(ui.lineEditFonction->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("IsoX");
contenu=doc.createTextNode(ui.lineEditIsoX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(ui.lineEditYmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(ui.lineEditYmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradY");
contenu=doc.createTextNode(ui.lineEditGradY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("IsoY");
contenu=doc.createTextNode(ui.lineEditIsoY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmin");
contenu=doc.createTextNode(ui.lineEditZmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Zmax");
contenu=doc.createTextNode(ui.lineEditZmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradZ");
contenu=doc.createTextNode(ui.lineEditGradZ->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("IsoZ");
contenu=doc.createTextNode(ui.lineEditIsoZ->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("RotX");
contenu=doc.createTextNode(QString::number(ui.spinBoxRotX->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("RotZ");
contenu=doc.createTextNode(QString::number(ui.spinBoxRotZ->value()));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Echelle");
contenu=doc.createTextNode(ui.lineEditEchelle->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("EchelleZ");
contenu=doc.createTextNode(ui.lineEditEchelleZ->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Points");
contenu=doc.createTextNode(pointList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Diagramme (*.pst)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Diagramme (*.pst)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Surface : ")+nomFichier);
}

void SurfaceDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Surface")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à des courbes"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Fonction");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditFonction->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("IsoX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditIsoX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("IsoY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditIsoY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Zmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditZmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Zmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditZmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradZ");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradZ->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("IsoZ");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditIsoZ->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("RotX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxRotX->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("RotZ");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.spinBoxRotZ->setValue(node.toElement().text().toInt());
	}

nodelist = doc.elementsByTagName("Echelle");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditEchelle->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("EchelleZ");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditEchelleZ->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Points");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	pointList=node.toElement().text().split("#");
	}

fichier.close();
ui.listWidgetPoints->clear();
for (int i = 0; i < pointList.count(); i++)
	{
	if (!pointList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetPoints);
		nouveauItem->setText(pointList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		pointList.removeAt(i);
		}
	}
if (ui.listWidgetPoints->count()>0)
	{
	ui.listWidgetPoints->setCurrentItem(ui.listWidgetPoints->item(0));
	ui.listWidgetPoints->setItemSelected(ui.listWidgetPoints->currentItem(), true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Surface : ")+nomFichier);
}

void SurfaceDialog::ChargerSurface()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Surface : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en charger un autre?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverSurface();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Diagramme (*.pst)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void SurfaceDialog::GnuplotTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+gnuplotTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << GenererCode();
fichier.close();
erreurProcessus=false;
gnuplotproc = new QProcess( this );
gnuplotproc->setWorkingDirectory(tempDir);
connect(gnuplotproc, SIGNAL(finished(int)),this, SLOT(finGnuplotProcess(int)));
gnuplotproc->start(gnuplotCommande+" "+gnuplotTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(QString::fromUtf8("Compilation en cours..."),0,0,3,this);
progressDialog->show();
progressDialog->raise();
progressDialog->setValue(0);
}

void SurfaceDialog::finGnuplotProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(3);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation Gnuplot"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(1);
	emit gnuplotfini();
	}
}

void SurfaceDialog::BboxTemp()
{
bboxproc = new QProcess( this );
bboxproc->setWorkingDirectory(tempDir);
connect(bboxproc, SIGNAL(finished(int)),this, SLOT(finBboxProcess(int)));
connect(bboxproc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
bboxproc->start(gsCommande+" -q -dBATCH -dNOPAUSE -sDEVICE=bbox "+psTempFile);
}

void SurfaceDialog::readInfoOutput()
{
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QString line;
QByteArray result=bboxproc->readAllStandardError();
QTextStream bbox(&result);
while ( !bbox.atEnd() ) 
	{
	line=bbox.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		int nx0=x0.toInt()-5;
		int nx1=x1.toInt()+5;
		int ny0=y0.toInt()-5;
		int ny1=y1.toInt()+5;
		QFile psf(tempDir+"/"+psTempFile);
		if (!psf.open( QIODevice::ReadOnly )) 
			{
			erreurProcessus=true;
			return;
			}
		QTextStream tpsf( &psf );
		line=tpsf.readAll();
		psf.close();
		line.replace(rxbbox,"%%BoundingBox: "+QString::number(nx0)+" "+QString::number(ny0)+" "+QString::number(nx1)+" "+QString::number(ny1));
		if (!psf.open(QIODevice::WriteOnly))
			{
			erreurProcessus=true;
			return;
			}
		QTextStream out( &psf );
		out << line;
		psf.close();
		emit bboxfini();
		}
	}
}


void SurfaceDialog::finBboxProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(3);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande bbox"));
	erreurProcessus=true;
	}
else if (progressDialog) progressDialog->setValue(2);
}

void SurfaceDialog::EpstopdfTemp()
{
bool ok;
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QFile psf(tempDir+"/"+psTempFile);
if (!psf.open( QIODevice::ReadOnly )) return;
QTextStream tpsf( &psf );
QString line;
while ( !tpsf.atEnd() ) 
	{
	line=tpsf.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		}
	}
psf.close();
float nx0=x0.toFloat(&ok);
float nx1=x1.toFloat(&ok);
if (nx1<=nx0) return;
float ny0=y0.toFloat(&ok);
float ny1=y1.toFloat(&ok);

epstopdfproc = new QProcess( this );
epstopdfproc->setWorkingDirectory(tempDir);
connect(epstopdfproc, SIGNAL(finished(int)),this, SLOT(finEpstopdfProcess(int)));
//epstopdfproc->start(epstopdfCommande+" "+psTempFile);
QString command=gsCommande+" -q -dBATCH -dNOPAUSE -dUseFlateCompression=false -sDEVICE=pdfwrite -sOutputFile="+pdfTempFile+" -c \"<< /PageSize ["+QString::number(nx1-nx0)+" "+QString::number(ny1-ny0)+"] >> setpagedevice\" "+QString::number(0-nx0)+" "+QString::number(0-ny0)+" translate -f "+psTempFile;
epstopdfproc->start(command);
}

void SurfaceDialog::finEpstopdfProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(3);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande epstopdf"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(3);
	emit epstopdffini();
	}
}


void SurfaceDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void SurfaceDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Surface : ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void SurfaceDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"surface");
aideDialog->exec();
}
//*****************************
void SurfaceDialog::ExporterEps()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterEpsSuite()));
GnuplotTemp();
}

void SurfaceDialog::ExporterEpsSuite()
{
if (!erreurProcessus)
	{
	QString epsFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Eps (*.eps)"));
	if (!epsFichier.isEmpty())
		{
		QFileInfo fi(epsFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+psTempFile,epsFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void SurfaceDialog::ExporterPdf()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterPdfSuite()));
GnuplotTemp();
}

void SurfaceDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void SurfaceDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

