/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tabvardialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QTableView>
#include <QTableWidgetItem>
#include <QColor>
#include <QFontMetrics>
#include <QUrl>
#include <QFontDatabase>


TabvarDialog::TabvarDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#ifndef Q_WS_MACX 
setModal(true);
#endif
//setWindowFlags(Qt::Window|Qt::WindowMinMaxButtonsHint);
connect(ui.quitButton, SIGNAL(clicked()), this, SLOT(accept()));
connect(ui.genererButton, SIGNAL(clicked()), this, SLOT(AfficherCode()));

connect(ui.tableWidget,SIGNAL(cellChanged(int,int)),this, SLOT(NettoyerCode()));
connect( ui.spinBoxLignes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserLignes(int)));
connect( ui.spinBoxColonnes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserColonnes(int)));

connect(ui.aideButton, SIGNAL(clicked()), this, SLOT(Aide()));

connect(ui.nouveauButton, SIGNAL(clicked()), this, SLOT(NouveauTableau()));
connect(ui.ouvrirButton, SIGNAL(clicked()), this, SLOT(ChargerTableau()));
connect(ui.sauverButton, SIGNAL(clicked()), this, SLOT(SauverTableau()));

connect(ui.apercuButton, SIGNAL(clicked()), this, SLOT(LatexTemp()));
connect(this, SIGNAL(latexfini()), this, SLOT(DvipsTemp()));
connect(this, SIGNAL(dvipsfini()), this, SLOT(BboxTemp()));
connect(this, SIGNAL(bboxfini()), this, SLOT(EpstopdfTemp()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));


connect(ui.epsButton, SIGNAL(clicked()), this, SLOT(ExporterEps()));
connect(ui.pdfButton, SIGNAL(clicked()), this, SLOT(ExporterPdf()));

ui.apercuButton->setShortcut(Qt::Key_F1);
ui.ouvrirButton->setShortcut(Qt::CTRL+Qt::Key_O);
ui.sauverButton->setShortcut(Qt::CTRL+Qt::Key_S);
ui.quitButton->setShortcut(Qt::CTRL+Qt::Key_Q);
ui.epsButton->setShortcut(Qt::CTRL+Qt::Key_E);
ui.pdfButton->setShortcut(Qt::CTRL+Qt::Key_P);
ui.apercuButton->setToolTip(QString::fromUtf8("F1"));
ui.ouvrirButton->setToolTip(QString::fromUtf8("CTRL+O"));
ui.sauverButton->setToolTip(QString::fromUtf8("CTRL+S"));
ui.quitButton->setToolTip(QString::fromUtf8("Quitter (CTRL+Q)"));
ui.epsButton->setToolTip(QString::fromUtf8("CTRL+E"));
ui.pdfButton->setToolTip(QString::fromUtf8("CTRL+P"));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pstplus_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
texTempFile=prefixFile+".tex";
dviTempFile=prefixFile+".dvi";
psTempFile=prefixFile+".eps";
pdfTempFile=prefixFile+".pdf";
pngTempFile=prefixFile+".png";
txtTempFile=prefixFile+".txt";

highlighter = new Highlighter(ui.textEditCode->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined( Q_WS_MACX )
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_WS_WIN)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
show();
}

TabvarDialog::~TabvarDialog(){
if (latexproc && latexproc->state()==QProcess::Running) 
	{
	latexproc->disconnect();
	latexproc->kill(); 
	delete latexproc ;
	}
if (dvipsproc && dvipsproc->state()==QProcess::Running) 
	{
	dvipsproc->disconnect();
	dvipsproc->kill(); 
	delete dvipsproc ;
	}
if (epstopdfproc && epstopdfproc->state()==QProcess::Running) 
	{
	epstopdfproc->disconnect();
	epstopdfproc->kill(); 
	delete epstopdfproc ;
	}
if (bboxproc && bboxproc->state()==QProcess::Running) 
	{
	bboxproc->disconnect();
	bboxproc->kill(); 
	delete bboxproc ;
	}
if (progressDialog) delete progressDialog;
}

void TabvarDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void TabvarDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void TabvarDialog::Init()
{
ui.textEditCode->clear();
ui.tableWidget->clear();
ui.spinBoxLignes->setValue(2);
ActualiserLignes(2);
ui.spinBoxColonnes->setValue(6);
ActualiserColonnes(6);
ui.tableWidget->setAcceptDrops(true);
ui.listWidgetMath->clear();
ui.listWidgetMath->setDragEnabled(true);
ui.listWidgetMath->addItems(QString("x;f(x);f^{\\,\\prime}(x);+;-;0;+\\infty;-\\infty").split(";"));
ui.listWidgetBarres->clear();
ui.listWidgetBarres->setDragEnabled(true);
ui.listWidgetBarres->addItems(QString("|;||").split(";"));
ui.listWidgetFleches->clear();
ui.listWidgetFleches->setDragEnabled(true);
ui.listWidgetFleches->addItems(QString("/;\\;_").split(";"));
}

void TabvarDialog::ActualiserLignes(int num)
{
QFontMetrics fontm=QFontMetrics(qApp->font());
int hauteur=fontm.lineSpacing()+4;
QStringList headerList;
for ( int i = 0; i <num; ++i )
	{
	headerList.append(QString::number(i+1)+" haut");
	headerList.append(QString::number(i+1)+" milieu");
	headerList.append(QString::number(i+1)+" bas");
	}
ui.tableWidget->setRowCount( num*3 );
ui.tableWidget->setVerticalHeaderLabels(headerList);
for ( int i = 0; i <num*3; ++i )
	{
	ui.tableWidget->setRowHeight(i,hauteur);
	}
ColorerTableau();
}

void TabvarDialog::ActualiserColonnes(int num)
{
QFontMetrics fontm=QFontMetrics(qApp->font());
int largeur=fontm.width( "wwwwwww" );
ui.tableWidget->setColumnCount( num );
for ( int i = 0; i <num; ++i )
	{
	ui.tableWidget->setColumnWidth(i,largeur);
	}
ColorerTableau();
}

void TabvarDialog::ColorerTableau()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item, *nouveau_item;
QColor couleurLigne;
QString contenu;
for ( int li=0;li<nblignes;li++) 
	{
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3, col,nouveau_item);
			}
		item=ui.tableWidget->item(li*3+1,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+1, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+1, col,nouveau_item);
			}
		item=ui.tableWidget->item(li*3+2,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+2, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+2, col,nouveau_item);
			}
		}
	}
}

void TabvarDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

QString TabvarDialog::GenererCode()
{
QString haut,milieu,bas;
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QString texcode="$$\\tabvar{%\n";
for ( int li=0;li<nblignes;li++) 
	{
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item) haut=item->text();
		else haut="";
		item=ui.tableWidget->item(li*3+1,col);
		if (item) milieu=item->text();
		else milieu="";
		item=ui.tableWidget->item(li*3+2,col);
		if (item) bas=item->text();
		else bas="";
		if (haut=="|")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\txbt{ }";
				else texcode+="\\txbt{"+bas+"}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\txt{ }";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\traithaut";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\txt{"+milieu+"}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="||")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\dbt";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\doubletraithaut";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="/")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fmh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="\\")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fdh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="_")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fhh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut.isEmpty())
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\txht{ }";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\traitmilieu";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\doubletraitmilieu";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fm";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fd";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fhm";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\traitbas";
				else if (bas=="||") texcode+="\\doubletraitbas";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\txb{"+bas+"}";
				}
			else
				{
				if (bas=="|") texcode+="\\xt{"+milieu+"}";
				else if (bas=="||") texcode+="\\xdbt{"+milieu+"}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{"+milieu+"}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\txht{"+haut+"}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\txh{"+haut+"}";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		if (col<nbcolonnes-1) texcode+="&";
		else texcode+="\\cr\n";
		}
	}
texcode+="}$$\n";
return texcode;
}

void TabvarDialog::AfficherCode()
{
QString texcode="%\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
texcode+="%\\input tabvar\n";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************
void TabvarDialog::LatexTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile tabvar(":/documents/tabvar.txt");
tabvar.open(QIODevice::ReadOnly);
QString preambule;
QTextStream in(&tabvar);
while (!in.atEnd()) 
	{
	preambule+= in.readLine()+"\n";
	}
tabvar.close();
QFile fichier;
fichier.setFileName(tempDir+"/"+texTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << "\\documentclass[10pt,a4paper]{article}\n";
out << "\\usepackage{pstricks,pst-plot,pst-text,pst-tree,pst-eps,pst-fill,pst-node,pst-math}\n";
out << "\\usepackage{amsmath}\n\\usepackage{amsfonts}\n\\usepackage{amssymb}\n";
out << preambule;
out << "\\pagestyle{empty}\n";
out << "\\begin{document}\n";
out << "\\TeXtoEPS\n";
out << GenererCode();
out << "\\endTeXtoEPS\n";
out << "\\end{document}\n";
fichier.close();
erreurProcessus=false;
latexproc = new QProcess( this );
latexproc->setWorkingDirectory(tempDir);
connect(latexproc, SIGNAL(finished(int)),this, SLOT(finLatexProcess(int)));
latexproc->start(latexCommande+" -interaction=nonstopmode "+texTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(QString::fromUtf8("Compilation en cours..."),0,0,4,this);
progressDialog->show();
progressDialog->raise();
progressDialog->setValue(0);
}

void TabvarDialog::finLatexProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	LogDialog *logDlg = new LogDialog(this);
	logDlg->exec();
	//QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation LaTeX"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(1);
	emit latexfini();
	}
}

void TabvarDialog::DvipsTemp()
{
dvipsproc = new QProcess( this );
dvipsproc->setWorkingDirectory(tempDir);
connect(dvipsproc, SIGNAL(finished(int)),this, SLOT(finDvipsProcess(int)));
dvipsproc->start(dvipsCommande+" -Ppk -V -E -o "+psTempFile+" "+dviTempFile);
}

void TabvarDialog::finDvipsProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande dvips"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(2);
	emit dvipsfini();
	}
}

void TabvarDialog::BboxTemp()
{
bboxproc = new QProcess( this );
bboxproc->setWorkingDirectory(tempDir);
connect(bboxproc, SIGNAL(finished(int)),this, SLOT(finBboxProcess(int)));
connect(bboxproc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
bboxproc->start(gsCommande+" -q -dBATCH -dNOPAUSE -sDEVICE=bbox "+psTempFile);
}

void TabvarDialog::readInfoOutput()
{
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QString line;
QByteArray result=bboxproc->readAllStandardError();
QTextStream bbox(&result);
while ( !bbox.atEnd() ) 
	{
	line=bbox.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		int nx0=x0.toInt()-5;
		int nx1=x1.toInt()+5;
		int ny0=y0.toInt()-5;
		int ny1=y1.toInt()+5;
		QFile psf(tempDir+"/"+psTempFile);
		if (!psf.open( QIODevice::ReadOnly )) 
			{
			erreurProcessus=true;
			return;
			}
		QTextStream tpsf( &psf );
		line=tpsf.readAll();
		psf.close();
		line.replace(rxbbox,"%%BoundingBox: "+QString::number(nx0)+" "+QString::number(ny0)+" "+QString::number(nx1)+" "+QString::number(ny1));
		if (!psf.open(QIODevice::WriteOnly))
			{
			erreurProcessus=true;
			return;
			}
		QTextStream out( &psf );
		out << line;
		psf.close();
		emit bboxfini();
		}
	}
}


void TabvarDialog::finBboxProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande bbox"));
	erreurProcessus=true;
	}
else if (progressDialog) progressDialog->setValue(3);
}


void TabvarDialog::EpstopdfTemp()
{
bool ok;
QRegExp rxbbox("%%BoundingBox:\\s*([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)\\s+([0-9eE\\.\\-]+)");
QString x0,y0,x1,y1;
QFile psf(tempDir+"/"+psTempFile);
if (!psf.open( QIODevice::ReadOnly )) return;
QTextStream tpsf( &psf );
QString line;
while ( !tpsf.atEnd() ) 
	{
	line=tpsf.readLine();
	if (rxbbox.indexIn(line)>-1)
		{
		x0=rxbbox.cap(1);
		y0=rxbbox.cap(2);
		x1=rxbbox.cap(3);
		y1=rxbbox.cap(4);
		}
	}
psf.close();
float nx0=x0.toFloat(&ok);
float nx1=x1.toFloat(&ok);
if (nx1<=nx0) return;
float ny0=y0.toFloat(&ok);
float ny1=y1.toFloat(&ok);

epstopdfproc = new QProcess( this );
epstopdfproc->setWorkingDirectory(tempDir);
connect(epstopdfproc, SIGNAL(finished(int)),this, SLOT(finEpstopdfProcess(int)));
//epstopdfproc->start(epstopdfCommande+" "+psTempFile);
QString command=gsCommande+" -q -dBATCH -dNOPAUSE -dUseFlateCompression=false -sDEVICE=pdfwrite -sOutputFile="+pdfTempFile+" -c \"<< /PageSize ["+QString::number(nx1-nx0)+" "+QString::number(ny1-ny0)+"] >> setpagedevice\" "+QString::number(0-nx0)+" "+QString::number(0-ny0)+" translate -f "+psTempFile;
epstopdfproc->start(command);
}

void TabvarDialog::finEpstopdfProcess(int err)
{
if (err) 
	{
	if (progressDialog) progressDialog->setValue(4);
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la commande epstopdf"));
	erreurProcessus=true;
	}
else 
	{
	if (progressDialog) progressDialog->setValue(4);
	emit epstopdffini();
	}
}


void TabvarDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void TabvarDialog::NouveauTableau()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Tableaux de variations : "),
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::SauverTableau()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QStringList haut,milieu,bas;

QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Tableau");
doc.appendChild(root);

description=doc.createElement("Lignes");
contenu=doc.createTextNode(QString::number(nblignes));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Colonnes");
contenu=doc.createTextNode(QString::number(nbcolonnes));
root.appendChild(description);
description.appendChild(contenu);


for ( int li=0;li<nblignes;li++) 
	{
	haut.clear();
	milieu.clear();
	bas.clear();
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item) 
			{
			if (!item->text().isEmpty()) haut.append(item->text());
			else haut.append("vide");
			}
		else haut.append("vide");
		item=ui.tableWidget->item(li*3+1,col);
		if (item) 
			{
			if (!item->text().isEmpty()) milieu.append(item->text());
			else milieu.append("vide");
			}
		else milieu.append("vide");
		item=ui.tableWidget->item(li*3+2,col);
		if (item) 
			{
			if (!item->text().isEmpty()) bas.append(item->text());
			else bas.append("vide");
			}
		else bas.append("vide");
		}
	description=doc.createElement("Haut");
	contenu=doc.createTextNode(haut.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);

	description=doc.createElement("Milieu");
	contenu=doc.createTextNode(milieu.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);

	description=doc.createElement("Bas");
	contenu=doc.createTextNode(bas.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);
	}

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Tableau (*.pst)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Tableau (*.pst)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;

QTableWidgetItem *nouveau_item;
QStringList haut,milieu,bas;
int nblignes,nbcolonnes;
QColor couleurLigne;
QString contenu;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Tableau")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un tableau"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Lignes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nblignes=node.toElement().text().toInt();
	}
nodelist = doc.elementsByTagName("Colonnes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nbcolonnes=node.toElement().text().toInt();
	}
ui.spinBoxLignes->setValue(nblignes);
ActualiserLignes(nblignes);
ui.spinBoxColonnes->setValue(nbcolonnes);
ActualiserColonnes(nbcolonnes);

nodelist = doc.elementsByTagName("Haut");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	haut=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < haut.count(); col++)
		{
		contenu=haut.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3, col,nouveau_item);
		}
	}

nodelist = doc.elementsByTagName("Milieu");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	milieu=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < milieu.count(); col++)
		{
		contenu=milieu.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3+1, col,nouveau_item);
		}
	}

nodelist = doc.elementsByTagName("Bas");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	bas=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < bas.count(); col++)
		{
		contenu=bas.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3+2, col,nouveau_item);
		}
	}

fichier.close();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::ChargerTableau()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,QString::fromUtf8("Tableau (*.pst)"));
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}

//*******************************
void TabvarDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"tabvar");
aideDialog->exec();
}

//*****************************
void TabvarDialog::ExporterEps()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterEpsSuite()));
LatexTemp();
}

void TabvarDialog::ExporterEpsSuite()
{
if (!erreurProcessus)
	{
	QString epsFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Eps (*.eps)"));
	if (!epsFichier.isEmpty())
		{
		QFileInfo fi(epsFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+psTempFile,epsFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void TabvarDialog::ExporterPdf()
{
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(ExporterPdfSuite()));
LatexTemp();
}

void TabvarDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(epstopdffini()));
connect(this, SIGNAL(epstopdffini()), this, SLOT(VoirTemp()));
}

void TabvarDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

