/***************************************************************************
 *   copyright       : (C) 2006-2011 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pstplus.h"

#include "courbesdialog.h"
#include "suitesdialog.h"
#include "histodialog.h"
#include "surfacedialog.h"
#include "tabvardialog.h"
#include "arbredialog.h"
#include "geometriedialog.h"
#include "graphedialog.h"
#include "boitesdialog.h"
#include "espacedialog.h"
#include "listdelegate.h" 

#if defined( Q_WS_X11 )
#include "x11fontdialog.h"
#endif

#include <QSettings>
#include <QAction>
#include <QStatusBar>
#include <QMessageBox>
#include <QApplication>
#include <QDir>
#include <QDesktopWidget>
#include <QStyleFactory>
#include <QStyle>
#include <QDomDocument>
#include <QDomElement>
#include <QFile>
#include <QFileDialog>
#include <QFileInfo>
#include <QFontDatabase>
#include <QDebug>
#include <QTextStream>
#include <QTextCodec>
#include <QColorDialog>

MainWindow::MainWindow(QWidget *parent)
    :QMainWindow( parent)
{
fichier="";
ui.setupUi(this);

#ifdef Q_WS_MACX
setWindowIcon(QIcon(":/images/logoapropos.png"));
#else
setWindowIcon(QIcon(":/images/logo22.png"));
#endif
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
QString contenu;
QFile apropos(":/documents/presentation.txt");
apropos.open(QIODevice::ReadOnly);
QTextStream in(&apropos);
in.setCodec(codec);
while (!in.atEnd()) 
	{
	contenu+= in.readLine()+"\n";
	}
apropos.close();
ui.textEdit->setHtml(contenu);

ui.textBrowser->setOpenExternalLinks(true);
ui.textBrowser->setHtml(QString::fromUtf8("<b>PST+ "PSTPLUSVERSION"</b><br>Assistants LaTeX/PSTricks pour documents mathématiques.<br><b>Copyright (c) 2006-2011 par Pascal Brachet</b><br><br>Site web : <a href=\"http://www.xm1math.net/pstplus/\">http://www.xm1math.net/pstplus/</a><br><br>Ce programme libre est distribué selon les termes de la licence GNU General Public License Version 2 (Free Software Foundation)<i><br><br>PST+ contient du code venant des programmes Eukleides (Copyright (c) Christian Obrecht), QtCreator (Copyright (C) Nokia - Licence: GPL), Texworks ((C) Jonathan Kew - Licence: GPL) et utilise le fichier tabvar.tex (Copyright (c) Denis Léger).<br>PST+ utilise aussi des icônes venant de l'environnement de bureau KDE ((C) KDE ev - Licence: GPL).<br>Merci à Olivier Hamant.</i>.<br><br>Ce programme a été réalisé sur un système d'exploitation Linux.<br>L'auteur de PST+ est aussi le créateur de <b><i>Texmaker</i></b>, éditeur LaTeX libre et gratuit fonctionnant sur les systèmes Linux, Macosx et Win32 (programme disponible à l'adresse suivante : <a href=\"http://www.xm1math.net/texmaker/\">http://www.xm1math.net/texmaker</a>)."));

QFile file(":/documents/styleapplication.qss");
file.open(QFile::ReadOnly);
QString styleSheet = QLatin1String(file.readAll());
qApp->setStyleSheet(styleSheet);

connect( ui.pushButtonLatex, SIGNAL(clicked()), this, SLOT(chercherLatex()));
connect( ui.pushButtonDvips, SIGNAL(clicked()), this, SLOT(chercherDvips()));
connect( ui.pushButtonLecteurpdf, SIGNAL(clicked()), this, SLOT(chercherLecteurpdf()));
connect( ui.pushButtonGhostscript, SIGNAL(clicked()), this, SLOT(chercherGhostscript()));
connect( ui.pushButtonGnuplot, SIGNAL(clicked()), this, SLOT(chercherGnuplot()));
connect( ui.pushButtonCouleur1, SIGNAL(clicked()), this, SLOT(configurerCouleur1()));
connect( ui.pushButtonCouleur2, SIGNAL(clicked()), this, SLOT(configurerCouleur2()));
connect( ui.pushButtonCouleur3, SIGNAL(clicked()), this, SLOT(configurerCouleur3()));

QFont commandFont=qApp->font();
QFont titleFont = commandFont;
titleFont.setBold(true);

QColor titleBg("#447BCD");
QColor titleFg("#ffffff");

ui.listWidget->setItemDelegate(new ListDelegate(ui.listWidget));

itemAnalyse = new QListWidgetItem(ui.listWidget);
itemAnalyse->setText(QString::fromUtf8("ANALYSE"));
itemAnalyse->setBackgroundColor(titleBg);
itemAnalyse->setTextColor(titleFg);
itemAnalyse->setFont(titleFont);
itemAnalyse->setTextAlignment(Qt::AlignHCenter);

itemCourbe = new QListWidgetItem(ui.listWidget);
itemCourbe->setText(QString::fromUtf8("Courbes 2D"));
itemCourbe->setFont(commandFont);
//itemCourbe->setSizeHint(QSize(225,25));

itemSuite = new QListWidgetItem(ui.listWidget);
itemSuite->setText(QString::fromUtf8("Suite récurrente"));
itemSuite->setFont(commandFont);
//itemSuite->setSizeHint(QSize(225,25));

itemTableau = new QListWidgetItem(ui.listWidget);
itemTableau->setText(QString::fromUtf8("Tableau de variations"));
itemTableau->setFont(commandFont);
//itemTableau->setSizeHint(QSize(225,25));

itemSurface = new QListWidgetItem(ui.listWidget);
itemSurface->setText(QString::fromUtf8("Surface 3D"));
itemSurface->setFont(commandFont);
//itemSurface->setSizeHint(QSize(225,25));

itemGeometrie = new QListWidgetItem(ui.listWidget);
itemGeometrie->setText(QString::fromUtf8("GÉOMETRIE"));
itemGeometrie->setBackgroundColor(titleBg);
itemGeometrie->setTextColor(titleFg);
itemGeometrie->setFont(titleFont);
itemGeometrie->setTextAlignment(Qt::AlignHCenter);

itemGeo = new QListWidgetItem(ui.listWidget);
itemGeo->setText(QString::fromUtf8("Figures géométriques"));
itemGeo->setFont(commandFont);
//itemGeo->setSizeHint(QSize(225,25));

itemEspace = new QListWidgetItem(ui.listWidget);
itemEspace->setText(QString::fromUtf8("Repère dans l'espace"));
itemEspace->setFont(commandFont);
//itemEspace->setSizeHint(QSize(225,25));

itemStat = new QListWidgetItem(ui.listWidget);
itemStat->setText(QString::fromUtf8("STATISTIQUE"));
itemStat->setBackgroundColor(titleBg);
itemStat->setTextColor(titleFg);
itemStat->setFont(titleFont);
itemStat->setTextAlignment(Qt::AlignHCenter);

itemBoite = new QListWidgetItem(ui.listWidget);
itemBoite->setText(QString::fromUtf8("Diagramme en boîtes"));
itemBoite->setFont(commandFont);
//itemBoite->setSizeHint(QSize(225,25));

itemHisto = new QListWidgetItem(ui.listWidget);
itemHisto->setText(QString::fromUtf8("Histogramme"));
itemHisto->setFont(commandFont);
//itemHisto->setSizeHint(QSize(225,25));

itemProba = new QListWidgetItem(ui.listWidget);
itemProba->setText(QString::fromUtf8("PROBABILITÉS"));
itemProba->setBackgroundColor(titleBg);
itemProba->setTextColor(titleFg);
itemProba->setFont(titleFont);
itemProba->setTextAlignment(Qt::AlignHCenter);

itemArbre = new QListWidgetItem(ui.listWidget);
itemArbre->setText(QString::fromUtf8("Arbres pondérés"));
itemArbre->setFont(commandFont);
//itemArbre->setSizeHint(QSize(225,25));

itemAutre = new QListWidgetItem(ui.listWidget);
itemAutre->setText(QString::fromUtf8("AUTRES"));
itemAutre->setBackgroundColor(titleBg);
itemAutre->setTextColor(titleFg);
itemAutre->setFont(titleFont);
itemAutre->setTextAlignment(Qt::AlignHCenter);

itemGraphe = new QListWidgetItem(ui.listWidget);
itemGraphe->setText(QString::fromUtf8("Graphes"));
itemGraphe->setFont(commandFont);
//itemGraphe->setSizeHint(QSize(225,25));

LireConfig();

ui.lineEditLatex->setText(latexCd);
ui.lineEditDvips->setText(dvipsCd);
ui.lineEditLecteurpdf->setText(lecteurpdfCd);
ui.lineEditGhostscript->setText(gsCd);
ui.lineEditGnuplot->setText(gnuplotCd);
ui.pushButtonCouleur1->setPalette(QPalette(couleur1));
ui.pushButtonCouleur1->setAutoFillBackground(true);
ui.pushButtonCouleur2->setPalette(QPalette(couleur2));
ui.pushButtonCouleur2->setAutoFillBackground(true);
ui.pushButtonCouleur3->setPalette(QPalette(couleur3));
ui.pushButtonCouleur3->setAutoFillBackground(true);
ui.checkBoxCadre->setChecked(affCadre);
ui.checkBoxSmall->setChecked(useSmall);

QAction *Act;

aslatMenu = menuBar()->addMenu(QString::fromUtf8("PST+"));
Act = new QAction(QIcon(":/images/configurer.png"), QString::fromUtf8("Configurer"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Configurer()));
aslatMenu->addAction(Act);
aslatMenu->addSeparator();
Act = new QAction(QString::fromUtf8("Ouvrir un exemple"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ChargerExemple()));
aslatMenu->addAction(Act);
aslatMenu->addSeparator();
#if defined( Q_WS_X11 )
Act = new QAction(QString::fromUtf8("Changer Police Interface"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(SetInterfaceFont()));
aslatMenu->addAction(Act);
aslatMenu->addSeparator();
#endif
Act = new QAction(QIcon(":/images/quitter16.png"), QString::fromUtf8("Quitter"), this);
Act->setShortcut(Qt::CTRL+Qt::Key_Q);
connect(Act, SIGNAL(triggered()), this, SLOT(Quitter()));
aslatMenu->addAction(Act);

assistantMenu = menuBar()->addMenu(QString::fromUtf8("Assistants"));
Act = new QAction(QString::fromUtf8("Courbes 2D"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Courbes()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Tableaux de variations"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Tabvar()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Arbres pondérés"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Arbre()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Figures géométriques"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Geometrie()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Graphes"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Graphe()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Diagramme en boîtes"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Boites()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Suite récurrente"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Suite()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Histogramme"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Histogramme()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Surface 3D"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Surface()));
assistantMenu->addAction(Act);
Act = new QAction(QString::fromUtf8("Repère dans l'espace"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Espace()));
assistantMenu->addAction(Act);

connect(ui.listWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(LancerAssistant(QListWidgetItem*)));
ui.tabWidget->setCurrentIndex(0);
}

MainWindow::~MainWindow(){
}

void MainWindow::closeEvent(QCloseEvent *e)
{
SauverConfig();
e->accept();
}

void MainWindow::Quitter()
{
SauverConfig();
qApp->quit();
}

void MainWindow::Ouvrir(QString f )
{
QDomDocument doc;
if (f.isEmpty()) return;
QFileInfo fi(f);
if (!fi.exists()) return;
QFile fich(f);
if (!fich.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fich)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fich.close();
	return;
	}
QDomElement root = doc.documentElement();
if (root.tagName() == "Courbes")
	{
	fich.close();
	fichier=f;
	Courbes();
	}
else if (root.tagName() == "Tableau")
	{
	fich.close();
	fichier=f;
	Tabvar();
	}
else if (root.tagName() == "Arbre")
	{
	fich.close();
	fichier=f;
	Arbre();
	}
else if (root.tagName() == "Geometrie")
	{
	fich.close();
	fichier=f;
	Geometrie();
	}
else if (root.tagName() == "Graphe")
	{
	fich.close();
	fichier=f;
	Graphe();
	}
else if (root.tagName() == "Boites")
	{
	fich.close();
	fichier=f;
	Boites();
	}
else if (root.tagName() == "Histogramme")
	{
	fich.close();
	fichier=f;
	Histogramme();
	}
else if (root.tagName() == "Suite")
	{
	fich.close();
	fichier=f;
	Suite();
	}
else if (root.tagName() == "Surface")
	{
	fich.close();
	fichier=f;
	Surface();
	}
else if (root.tagName() == "Espace")
	{
	fich.close();
	fichier=f;
	Espace();
	}
else
	{
	fich.close();
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Le fichier ne correspond à aucun des assistants"));
	}

}

//*****************************************
void MainWindow::Courbes()
{
ActualiserConfig();
CourbesDialog *courbesDialog = new CourbesDialog(this,useSmall);
courbesDialog->latexCommande=latexCd;
courbesDialog->dvipsCommande=dvipsCd;
courbesDialog->gsCommande=gsCd;
courbesDialog->lecteurpdfCommande=lecteurpdfCd;
courbesDialog->dernierRepertoire=dernierRp;
courbesDialog->couleurSupp1=couleur1;
courbesDialog->couleurSupp2=couleur2;
courbesDialog->couleurSupp3=couleur3;
courbesDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) courbesDialog->Ouvrir(fichier);
if (courbesDialog->exec()) dernierRp=courbesDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Tabvar()
{
ActualiserConfig();
TabvarDialog *tabvarDialog = new TabvarDialog(this);
tabvarDialog->latexCommande=latexCd;
tabvarDialog->dvipsCommande=dvipsCd;
tabvarDialog->gsCommande=gsCd;
tabvarDialog->lecteurpdfCommande=lecteurpdfCd;
tabvarDialog->dernierRepertoire=dernierRp;
tabvarDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) tabvarDialog->Ouvrir(fichier);
if (tabvarDialog->exec()) dernierRp=tabvarDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Arbre()
{
ActualiserConfig();
ArbreDialog *arbreDialog = new ArbreDialog(this,useSmall);
arbreDialog->latexCommande=latexCd;
arbreDialog->dvipsCommande=dvipsCd;
arbreDialog->gsCommande=gsCd;
arbreDialog->lecteurpdfCommande=lecteurpdfCd;
arbreDialog->dernierRepertoire=dernierRp;
arbreDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) arbreDialog->Ouvrir(fichier);
if (arbreDialog->exec()) dernierRp=arbreDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Geometrie()
{
ActualiserConfig();
GeometrieDialog *geoDialog = new GeometrieDialog(this,useSmall);
geoDialog->latexCommande=latexCd;
geoDialog->dvipsCommande=dvipsCd;
geoDialog->gsCommande=gsCd;
geoDialog->lecteurpdfCommande=lecteurpdfCd;
geoDialog->dernierRepertoire=dernierRp;
geoDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) geoDialog->Ouvrir(fichier);
if (geoDialog->exec()) dernierRp=geoDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Graphe()
{
ActualiserConfig();
GrapheDialog *grapheDialog = new GrapheDialog(this);
grapheDialog->latexCommande=latexCd;
grapheDialog->dvipsCommande=dvipsCd;
grapheDialog->gsCommande=gsCd;
grapheDialog->lecteurpdfCommande=lecteurpdfCd;
grapheDialog->dernierRepertoire=dernierRp;
grapheDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) grapheDialog->Ouvrir(fichier);
if (grapheDialog->exec()) dernierRp=grapheDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Boites()
{
ActualiserConfig();
BoitesDialog *boitesDialog = new BoitesDialog(this);
boitesDialog->latexCommande=latexCd;
boitesDialog->dvipsCommande=dvipsCd;
boitesDialog->gsCommande=gsCd;
boitesDialog->lecteurpdfCommande=lecteurpdfCd;
boitesDialog->dernierRepertoire=dernierRp;
boitesDialog->couleurSupp1=couleur1;
boitesDialog->couleurSupp2=couleur2;
boitesDialog->couleurSupp3=couleur3;
boitesDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) boitesDialog->Ouvrir(fichier);
if (boitesDialog->exec()) dernierRp=boitesDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Suite()
{
ActualiserConfig();
SuitesDialog *suitesDialog = new SuitesDialog(this,useSmall);
suitesDialog->latexCommande=latexCd;
suitesDialog->dvipsCommande=dvipsCd;
suitesDialog->gsCommande=gsCd;
suitesDialog->lecteurpdfCommande=lecteurpdfCd;
suitesDialog->dernierRepertoire=dernierRp;
suitesDialog->couleurSupp1=couleur1;
suitesDialog->couleurSupp2=couleur2;
suitesDialog->couleurSupp3=couleur3;
suitesDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) suitesDialog->Ouvrir(fichier);
if (suitesDialog->exec()) dernierRp=suitesDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Histogramme()
{
ActualiserConfig();
HistoDialog *histoDialog = new HistoDialog(this);
histoDialog->latexCommande=latexCd;
histoDialog->dvipsCommande=dvipsCd;
histoDialog->gsCommande=gsCd;
histoDialog->lecteurpdfCommande=lecteurpdfCd;
histoDialog->dernierRepertoire=dernierRp;
histoDialog->couleurSupp1=couleur1;
histoDialog->couleurSupp2=couleur2;
histoDialog->couleurSupp3=couleur3;
histoDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) histoDialog->Ouvrir(fichier);
if (histoDialog->exec()) dernierRp=histoDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Surface()
{
ActualiserConfig();
SurfaceDialog *surfaceDialog = new SurfaceDialog(this);
surfaceDialog->gnuplotCommande=gnuplotCd;
surfaceDialog->gsCommande=gsCd;
surfaceDialog->lecteurpdfCommande=lecteurpdfCd;
surfaceDialog->dernierRepertoire=dernierRp;
surfaceDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) surfaceDialog->Ouvrir(fichier);
if (surfaceDialog->exec()) dernierRp=surfaceDialog->dernierRepertoire;
fichier="";
}

void MainWindow::Espace()
{
ActualiserConfig();
EspaceDialog *espaceDialog = new EspaceDialog(this,useSmall);
espaceDialog->latexCommande=latexCd;
espaceDialog->dvipsCommande=dvipsCd;
espaceDialog->gsCommande=gsCd;
espaceDialog->lecteurpdfCommande=lecteurpdfCd;
espaceDialog->dernierRepertoire=dernierRp;
espaceDialog->couleurSupp1=couleur1;
espaceDialog->couleurSupp2=couleur2;
espaceDialog->couleurSupp3=couleur3;
espaceDialog->afficherCadre=affCadre;
if (!fichier.isEmpty()) espaceDialog->Ouvrir(fichier);
if (espaceDialog->exec()) dernierRp=espaceDialog->dernierRepertoire;
fichier="";
}

//*************************
void MainWindow::LireConfig()
{
#ifdef USB_VERSION
QSettings *config=new QSettings(QCoreApplication::applicationDirPath()+"/pstplus.ini",QSettings::IniFormat); //for USB-stick version :
#else
QSettings *config=new QSettings(QSettings::IniFormat,QSettings::UserScope,"xm1","pstplus");
#endif
/*if (!config->contains("IniMode")) 
	{
	delete config;
	config=new QSettings("xm1","pstplus");
	}*/

config->beginGroup( "pstplus" );
#ifdef Q_WS_MACX
latexCd=config->value("commande latex","\"/usr/texbin/latex\"").toString();
dvipsCd=config->value("commande dvips","\"/usr/texbin/dvips\"").toString();
gsCd=config->value("commande ghostscript","/usr/local/bin/gs").toString();
lecteurpdfCd=config->value("commande lecteurpdf","open").toString();
gnuplotCd=config->value("commande gnuplot","gnuplot").toString();
#endif
#ifdef Q_WS_X11
int desktop_env=1; // 1 : gnome ; 2: kde ; 3 : kde4 ; 
QStringList styles = QStyleFactory::keys();
QString kdesession= ::getenv("KDE_FULL_SESSION");
QString kdeversion= ::getenv("KDE_SESSION_VERSION");
if (!kdesession.isEmpty()) desktop_env=2;
if (!kdeversion.isEmpty()) desktop_env=3;

switch (desktop_env)
	{
	case 1:
		{
		lecteurpdfCd=config->value("commande lecteurpdf","evince").toString();
		}break;
	case 2:
		{
		lecteurpdfCd=config->value("commande lecteurpdf","kpdf").toString();
		}break;
	case 3:
		{
		lecteurpdfCd=config->value("commande lecteurpdf","okular").toString();
		}break;
	}

latexCd=config->value("commande latex","latex").toString();
dvipsCd=config->value("commande dvips","dvips").toString();
gsCd=config->value("commande ghostscript","gs").toString();
gnuplotCd=config->value("commande gnuplot","gnuplot").toString();
x11style=config->value( "X11/Style","Plastique").toString();
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("DejaVu Sans",Qt::CaseInsensitive)) deft="DejaVu Sans";
else if (xf.contains("DejaVu Sans LGC",Qt::CaseInsensitive)) deft="DejaVu Sans LGC";
else if (xf.contains("Bitstream Vera Sans",Qt::CaseInsensitive)) deft="Bitstream Vera Sans";
else if (xf.contains("Luxi Sans",Qt::CaseInsensitive)) deft="Luxi Sans";
else deft=qApp->font().family();
x11fontfamily=config->value("X11/Font Family",deft).toString();
x11fontsize=config->value( "X11/Font Size","10").toInt();

if (desktop_env ==1) //no-kde
    {
    if (styles.contains("GTK+")) QApplication::setStyle(QStyleFactory::create("GTK+"));//gtkstyle
    else QApplication::setStyle(QStyleFactory::create("Cleanlooks"));
    }
else if ((desktop_env ==3) && (styles.contains("Oxygen"))) QApplication::setStyle(QStyleFactory::create("Oxygen")); //kde4+oxygen
else QApplication::setStyle(QStyleFactory::create("Plastique")); //others
QString baseStyleName = qApp->style()->objectName(); //fallback
if (baseStyleName == QLatin1String("Windows")) QApplication::setStyle(QStyleFactory::create("Plastique")); 


QFont x11Font (x11fontfamily,x11fontsize);
QApplication::setFont(x11Font);

QPalette pal = QApplication::palette();

pal.setColor( QPalette::Active, QPalette::Highlight, QColor("#4490d8") );
pal.setColor( QPalette::Inactive, QPalette::Highlight, QColor("#4490d8") );
pal.setColor( QPalette::Disabled, QPalette::Highlight, QColor("#4490d8") );


//pal.setColor( QPalette::Active, QPalette::Highlight, QColor("#5689be") );
//pal.setColor( QPalette::Inactive, QPalette::Highlight, QColor("#5689be") );
//pal.setColor( QPalette::Disabled, QPalette::Highlight, QColor("#5689be") );

pal.setColor( QPalette::Active, QPalette::HighlightedText, QColor("#ffffff") );
pal.setColor( QPalette::Inactive, QPalette::HighlightedText, QColor("#ffffff") );
pal.setColor( QPalette::Disabled, QPalette::HighlightedText, QColor("#ffffff") );

pal.setColor( QPalette::Active, QPalette::Base, QColor("#ffffff") );
pal.setColor( QPalette::Inactive, QPalette::Base, QColor("#ffffff") );
pal.setColor( QPalette::Disabled, QPalette::Base, QColor("#ffffff") );

pal.setColor( QPalette::Active, QPalette::WindowText, QColor("#000000") );
pal.setColor( QPalette::Inactive, QPalette::WindowText, QColor("#000000") );
pal.setColor( QPalette::Disabled, QPalette::WindowText, QColor("#000000") );

pal.setColor( QPalette::Active, QPalette::ButtonText, QColor("#000000") );
pal.setColor( QPalette::Inactive, QPalette::ButtonText, QColor("#000000") );
pal.setColor( QPalette::Disabled, QPalette::ButtonText, QColor("#000000") );

if (desktop_env ==3)
	{
	pal.setColor( QPalette::Active, QPalette::Window, QColor("#eae9e9") );
	pal.setColor( QPalette::Inactive, QPalette::Window, QColor("#eae9e9") );
	pal.setColor( QPalette::Disabled, QPalette::Window, QColor("#eae9e9") );

	pal.setColor( QPalette::Active, QPalette::Button, QColor("#eae9e9") );
	pal.setColor( QPalette::Inactive, QPalette::Button, QColor("#eae9e9") );
	pal.setColor( QPalette::Disabled, QPalette::Button, QColor("#eae9e9") );
	}
else
	{
/*
	pal.setColor( QPalette::Active, QPalette::Window, QColor("#f1f1f1") );
	pal.setColor( QPalette::Inactive, QPalette::Window, QColor("#f1f1f1") );
	pal.setColor( QPalette::Disabled, QPalette::Window, QColor("#f1f1f1") );

	pal.setColor( QPalette::Active, QPalette::Button, QColor("#f1f1f1") );
	pal.setColor( QPalette::Inactive, QPalette::Button, QColor("#f1f1f1") );
	pal.setColor( QPalette::Disabled, QPalette::Button, QColor("#f1f1f1") );
*/
	pal.setColor( QPalette::Active, QPalette::Window, QColor("#fbf8f1") );
	pal.setColor( QPalette::Inactive, QPalette::Window, QColor("#fbf8f1") );
	pal.setColor( QPalette::Disabled, QPalette::Window, QColor("#fbf8f1") );

	pal.setColor( QPalette::Active, QPalette::Button, QColor("#fbf8f1") );
	pal.setColor( QPalette::Inactive, QPalette::Button, QColor("#fbf8f1") );
	pal.setColor( QPalette::Disabled, QPalette::Button, QColor("#fbf8f1") );
	}

QApplication::setPalette(pal);
#endif
#ifdef Q_WS_WIN
latexCd=config->value("commande latex","latex").toString();
dvipsCd=config->value("commande dvips","dvips").toString();
gsCd=config->value("commande ghostscript","\"C:/Program Files/gs/gs9.02/bin/gswin32c.exe\"").toString();
QString gswin=gsCd;
gswin.remove("\"");
  if (!QFileInfo(gswin).exists())
    {
    if (QFileInfo("C:/Program Files (x86)/gs/gs9.02/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs9.02/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files/gs/gs9.00/bin/gswin32c.exe").exists()) gswin="C:/Program Files/gs/gs9.00/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files (x86)/gs/gs9.00/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs9.00/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files/gs/gs8.71/bin/gswin32c.exe").exists()) gswin="C:/Program Files/gs/gs8.71/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files (x86)/gs/gs8.71/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs8.71/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files/gs/gs8.64/bin/gswin32c.exe").exists()) gswin="C:/Program Files/gs/gs8.64/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files (x86)/gs/gs8.64/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs8.65/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files/gs/gs8.63/bin/gswin32c.exe").exists()) gswin="C:/Program Files/gs/gs8.63/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files (x86)/gs/gs8.63/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs8.63/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files/gs/gs8.62/bin/gswin32c.exe").exists()) gswin="C:/Program Files/gs/gs8.62/bin/gswin32c.exe";
    else if (QFileInfo("C:/Program Files (x86)/gs/gs8.62/bin/gswin32c.exe").exists()) gswin="C:/Program Files (x86)/gs/gs8.62/bin/gswin32c.exe";
    }
gsCd="\""+gswin+"\"";
lecteurpdfCd=config->value("commande lecteurpdf","\"C:/Program Files/Adobe/Reader 9.0/Reader/AcroRd32.exe\"").toString();
QString acro=lecteurpdfCd;
acro.remove("\"");
 if (!QFileInfo(acro).exists())
    {
    if (QFileInfo("C:/Program Files (x86)/Adobe/Reader 9.0/Reader/AcroRd32.exe").exists()) acro="C:/Program Files (x86)/Adobe/Reader 9.0/Reader/AcroRd32.exe";
    else if (QFileInfo("C:/Program Files/Adobe/Reader 8.0/Reader/AcroRd32.exe").exists()) acro="C:/Program Files/Adobe/Reader 8.0/Reader/AcroRd32.exe";
    else if (QFileInfo("C:/Program Files (x86)/Adobe/Reader 8.0/Reader/AcroRd32.exe").exists()) acro="C:/Program Files (x86)/Adobe/Reader 8.0/Reader/AcroRd32.exe";
    }
lecteurpdfCd="\""+acro+"\""; 
gnuplotCd=config->value("commande gnuplot","\"C:/Program Files/gnuplot/bin/wgnuplot.exe\"").toString();
#endif
dernierRp=config->value("dernier repertoire",QDir::homePath()).toString();
QRect screen = QApplication::desktop()->screenGeometry();
int x= config->value( "Geometries/MainwindowX",10).toInt();
int y= config->value( "Geometries/MainwindowY",10).toInt() ;
move(x,y);
couleur1=config->value("couleur utilisateur1",QColor(167,167,0)).value<QColor>();
couleur2=config->value("couleur utilisateur2",QColor(0,81,121)).value<QColor>();
couleur3=config->value("couleur utilisateur3",QColor(24,165,5)).value<QColor>();
affCadre=config->value( "afficher cadre",true).toBool();
#ifdef Q_WS_MACX
useSmall=config->value( "utiliser small",false).toBool();
#else
useSmall=config->value( "utiliser small",true).toBool();
#endif
config->endGroup();
}

void MainWindow::SauverConfig()
{
#ifdef USB_VERSION
QSettings config(QCoreApplication::applicationDirPath()+"/pstplus.ini",QSettings::IniFormat); //for USB-stick version 
#else
QSettings config(QSettings::IniFormat,QSettings::UserScope,"xm1","pstplus");
#endif
ActualiserConfig();

config.setValue( "IniMode",true);
config.beginGroup( "pstplus" );
config.setValue("commande latex",latexCd);
config.setValue("commande dvips",dvipsCd);
config.setValue("commande ghostscript",gsCd);
config.setValue("commande lecteurpdf",lecteurpdfCd);
config.setValue("commande gnuplot",gnuplotCd);
config.setValue("dernier repertoire",dernierRp);
config.setValue("Geometries/MainwindowX", x() );
config.setValue("Geometries/MainwindowY", y() );
#if defined( Q_WS_X11 )
config.setValue( "X11/Style",x11style);
config.setValue("X11/Font Family",x11fontfamily);
config.setValue( "X11/Font Size",x11fontsize);
#endif
config.setValue("couleur utilisateur1",couleur1);
config.setValue("couleur utilisateur2",couleur2);
config.setValue("couleur utilisateur3",couleur3);
config.setValue("afficher cadre",affCadre);
config.setValue("utiliser small",useSmall);
config.endGroup();
}

void MainWindow::Configurer()
{
ui.tabWidget->setCurrentIndex(1);
}

void MainWindow::ActualiserConfig()
{
latexCd=ui.lineEditLatex->text();
dvipsCd=ui.lineEditDvips->text();
lecteurpdfCd=ui.lineEditLecteurpdf->text();
gsCd=ui.lineEditGhostscript->text();
gnuplotCd=ui.lineEditGnuplot->text();
couleur1=ui.pushButtonCouleur1->palette().background().color();
couleur2=ui.pushButtonCouleur2->palette().background().color();
couleur3=ui.pushButtonCouleur3->palette().background().color();
affCadre=ui.checkBoxCadre->isChecked();
useSmall=ui.checkBoxSmall->isChecked();
}
//**************************
void MainWindow::ChargerExemple()
{
QString exempleRepertoire="";
#if defined( Q_WS_X11 )
#ifdef USB_VERSION
exempleRepertoire=QCoreApplication::applicationDirPath();
#else
exempleRepertoire=PREFIX"/share/pstplus";
#endif
#endif
#if defined( Q_WS_MACX )
exempleRepertoire=QCoreApplication::applicationDirPath()+"/../Resources";
//exempleRepertoire="/Applications/pstplus.app/Contents/Resources";
#endif
#if defined(Q_WS_WIN)
exempleRepertoire=QCoreApplication::applicationDirPath();
#endif
QString f="";
f = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),exempleRepertoire,"Fichier PST+ (*.pst)");
Ouvrir(f);
}
//*************************
void MainWindow::SetInterfaceFont()
{
#if defined( Q_WS_X11 )
X11FontDialog *xfdlg = new X11FontDialog(this);
int ft=xfdlg->ui.comboBoxFont->findText (x11fontfamily , Qt::MatchExactly);
xfdlg->ui.comboBoxFont->setCurrentIndex(ft);
xfdlg->ui.spinBoxSize->setValue(x11fontsize);
if (xfdlg->exec())
	{
	x11fontfamily=xfdlg->ui.comboBoxFont->currentText();
	x11fontsize=xfdlg->ui.spinBoxSize->value();
	QFont x11Font (x11fontfamily,x11fontsize);
	QApplication::setFont(x11Font);
	}
#endif
}
//*************************
void MainWindow::chercherLatex()
{
QString location=QFileDialog::getOpenFileName(this,QString::fromUtf8("Chercher programme"),QDir::rootPath(),"Programme (*)",0,QFileDialog::DontResolveSymlinks);
if ( !location.isEmpty() ) 
	{
	location.replace(QString("\\"),QString("/"));
	location="\""+location+"\"";
	ui.lineEditLatex->setText( location );
	}
}

void MainWindow::chercherDvips()
{
QString location=QFileDialog::getOpenFileName(this,QString::fromUtf8("Chercher programme"),QDir::rootPath(),"Programme (*)",0,QFileDialog::DontResolveSymlinks);
if ( !location.isEmpty() ) 
	{
	location.replace(QString("\\"),QString("/"));
	location="\""+location+"\"";
	ui.lineEditDvips->setText( location );
	}
}

void MainWindow::chercherLecteurpdf()
{
QString location=QFileDialog::getOpenFileName(this,QString::fromUtf8("Chercher programme"),QDir::rootPath(),"Programme (*)",0,QFileDialog::DontResolveSymlinks);
if ( !location.isEmpty() ) 
	{
	location.replace(QString("\\"),QString("/"));
	location="\""+location+"\"";
	ui.lineEditLecteurpdf->setText( location );
	}
}

void MainWindow::chercherGhostscript()
{
QString location=QFileDialog::getOpenFileName(this,QString::fromUtf8("Chercher programme"),QDir::rootPath(),"Programme (*)",0,QFileDialog::DontResolveSymlinks);
if ( !location.isEmpty() ) 
	{
	location.replace(QString("\\"),QString("/"));
	location="\""+location+"\"";
	ui.lineEditGhostscript->setText( location );
	}
}

void MainWindow::chercherGnuplot()
{
QString location=QFileDialog::getOpenFileName(this,QString::fromUtf8("Chercher programme"),QDir::rootPath(),"Programme (*)",0,QFileDialog::DontResolveSymlinks);
if ( !location.isEmpty() ) 
	{
	location.replace(QString("\\"),QString("/"));
	location="\""+location+"\"";
	ui.lineEditGnuplot->setText( location );
	}
}

void MainWindow::configurerCouleur1()
{
QColor color = QColorDialog::getColor(ui.pushButtonCouleur1->palette().background().color(), this);
if (color.isValid()) 
	{
	ui.pushButtonCouleur1->setPalette(QPalette(color));
	ui.pushButtonCouleur1->setAutoFillBackground(true);
	}
}

void MainWindow::configurerCouleur2()
{
QColor color = QColorDialog::getColor(ui.pushButtonCouleur2->palette().background().color(), this);
if (color.isValid()) 
	{
	ui.pushButtonCouleur2->setPalette(QPalette(color));
	ui.pushButtonCouleur2->setAutoFillBackground(true);
	}
}

void MainWindow::configurerCouleur3()
{
QColor color = QColorDialog::getColor(ui.pushButtonCouleur3->palette().background().color(), this);
if (color.isValid()) 
	{
	ui.pushButtonCouleur3->setPalette(QPalette(color));
	ui.pushButtonCouleur3->setAutoFillBackground(true);
	}
}

void MainWindow::LancerAssistant(QListWidgetItem* item)
{
if (item==itemCourbe) Courbes();
else if (item==itemSuite) Suite();
else if (item==itemTableau) Tabvar(); 
else if (item==itemSurface) Surface(); 
else if (item==itemGeo) Geometrie();
else if (item==itemEspace) Espace(); 
else if (item==itemBoite) Boites(); 
else if (item==itemHisto) Histogramme(); 
else if (item==itemArbre) Arbre();
else if (item==itemGraphe) Graphe(); 
}


